//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceBxxd {

//------------------------------------------------------------------------------
//  main
//------------------------------------------------------------------------------
public static void main(String[] argv) {
     SpaceServer server = null;
          int port = SpaceServer.defaultPort;

     try {
          //--------------------------------------------------------------------
          //  parse the command line
          //--------------------------------------------------------------------
          String configPathname = null;
          boolean inetd = true;

          for(int k=0;k<argv.length;++k) {
               //---------------------------------------------------------------
               //  -portno
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-portno")) {
                    port = InvisibleUtil.parseInt(argv[++k]);
                    continue;
                    }

               //---------------------------------------------------------------
               //  -inetd
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-inetd")) {
                    inetd = true;
                    continue;
                    }

               //---------------------------------------------------------------
               //  -standalone
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-standalone")) {
                    inetd = false;
                    continue;
                    }

               //---------------------------------------------------------------
               //  configuration pathname
               //---------------------------------------------------------------
               if(configPathname != null) {
                    return;
                    }

               configPathname = argv[k];
               }

          //--------------------------------------------------------------------
          //  load configuration file
          //--------------------------------------------------------------------
          SpaceConfigCache configCache = new SpaceConfigCache();

          configCache.load(configPathname);

          //--------------------------------------------------------------------
          //  get configuration for server port
          //--------------------------------------------------------------------
          SpaceConfig config = null;

          if((port == SpaceServer.defaultPort) && (inetd)) {
               port = configCache.getDefaultPort();
               config = configCache.getDefault();
               }
          else {
               config = configCache.get(port);
               }

          if(config == null) {
               System.err.println("No Configuration For Port " + port);
               return;
               }

          //--------------------------------------------------------------------
          //  open log file
          //--------------------------------------------------------------------
          SpaceLog log = new SpaceLog(config,port);

          //--------------------------------------------------------------------
          //  server loop
          //--------------------------------------------------------------------
          while(true) {
               //---------------------------------------------------------------
               //  start the server
               //---------------------------------------------------------------
               server = new SpaceServer(port);

               server.setLog(log);

               server.log(SpaceLog.info,"listen",port);

               server.accept(inetd);

               server.log(SpaceLog.info,"connect "+server.getRemoteAddress(),port);

               //---------------------------------------------------------------
               //  send the greeting
               //---------------------------------------------------------------
               String greeting = config.getGreeting();

               server.rsp(server.getChannelZero(),greeting,0);

               //---------------------------------------------------------------
               //  parse client greeting
               //---------------------------------------------------------------
               SpaceResponse clientGreeting = server.getChannelZeroResponse();

               server.wait(clientGreeting);

               server.log(SpaceLog.info,"greeting " +clientGreeting.toString(),port);

               server.parseGreeting(clientGreeting);

               //---------------------------------------------------------------
               //  get preferred localization
               //---------------------------------------------------------------
               String localize = server.getLocalize();

               server.log(SpaceLog.info,"localize " +localize,port);

               //---------------------------------------------------------------
               //  instantiate an empty user cache
               //---------------------------------------------------------------
               SpaceUserCache userCache = new SpaceUserCache();

               //---------------------------------------------------------------
               //  process requests
               //---------------------------------------------------------------
               while(true) {
                    //----------------------------------------------------------
                    //  get a request
                    //----------------------------------------------------------
                    SpaceRequest request = server.getRequest();

                    if(request != null) {
                         //-----------------------------------------------------
                         //  see if shutting down
                         //-----------------------------------------------------
                         if(request.length() == 0) {
                              server.log(SpaceLog.info,"disconnect "+server.getRemoteAddress(),port);
                              break;
                              }

                         //-----------------------------------------------------
                         //  see if channel start
                         //-----------------------------------------------------
                         request.parse();

                         //-----------------------------------------------------
                         //  instantiate the module
                         //-----------------------------------------------------
                         String profileName = request.getProfile();

                         server.log(SpaceLog.info,"profile " + profileName);

                         SpaceProfile profile = config.getProfile(profileName);

                         if(profile == null) {
                              server.log(SpaceLog.error,"unknown profile "+profileName,request);
                              return;
                              }

                         //-----------------------------------------------------
                         //  instantiate the module
                         //-----------------------------------------------------
                         String moduleName = profile.get("module");

                         if(moduleName == null) {
                              server.log(SpaceLog.error,"unknown module name "+moduleName,request);
                              return;
                              }

                         server.log(SpaceLog.info,"module "+moduleName,request);

                         SpaceModule module = SpaceModuleLoad.getClass(moduleName);

                         if(module == null) {
                              server.log(SpaceLog.error,"unknown module "+moduleName,request);
                              return;
                              }

                         //-----------------------------------------------------
                         //  start the module thread
                         //-----------------------------------------------------
                         module.setConfig(config);
                         module.setUserCache(userCache);
                         module.setProfile(profile);
                         module.setServer(server);
                         module.setRequest(request);

                         server.log(SpaceLog.info,"start",request);

                         module.start();
                         }

                    //----------------------------------------------------------
                    //  wait for a potential request
                    //----------------------------------------------------------
                    server.threadWait(2000);
                    }

               //---------------------------------------------------------------
               //  close
               //---------------------------------------------------------------
               server.close();
               server = null;

               //---------------------------------------------------------------
               //  exit if inetd
               //---------------------------------------------------------------
               if(inetd) break;
               }
          }
     catch(ArrayIndexOutOfBoundsException e) {
          System.err.println("Usage: java SpaceBxxd [-portno port] config_file");
          }
     catch(NullPointerException e) {
          System.err.println("Usage: java SpaceBxxd [-portno port] config_file");
          }
     catch(SpaceTimeout e) {
          try {
               server.log(SpaceLog.fatal,e.toString(),port);
               }
          catch(SpaceException e2) {
               }
          }
     catch(SpaceException e) {
          try {
               server.log(SpaceLog.fatal,e.toString(),port);
               }
          catch(SpaceException e2) {
               }
          }
     finally {
          if(server != null) {
               server.close();
               }
          }
     }
}
