//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceModuleSASLOTP extends SpaceModuleThread {

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceModuleSASLOTP() {
     super();
     }

//------------------------------------------------------------------------------
//  run
//------------------------------------------------------------------------------
public void run() {
     try {
          //--------------------------------------------------------------------
          //  see if channel start
          //--------------------------------------------------------------------
          if(isStart()) {
               //---------------------------------------------------------------
               //  get the authenticator
               //---------------------------------------------------------------
               //String authenticator = request.getAuthenticator();
               String authenticator = InvisibleBase64.decode(request.getAuthenticator());

               if(authenticator.charAt(0) != 0) {
                    error(551);
                    return;
                    }

               authenticator = authenticator.substring(1);

               server.setAuthenticator(authenticator);

               //---------------------------------------------------------------
               //  load usernames
               //---------------------------------------------------------------
               loadUser();

               //---------------------------------------------------------------
               //  get and verify the challenge elements
               //---------------------------------------------------------------
               String mechanism = getUser(authenticator,"mechanism");
               String algorithm = getUser(authenticator,"algorithm");
               String sequence = getUser(authenticator,"sequence");
               String seed = getUser(authenticator,"seed");

               if((mechanism == null) || (algorithm == null) || (sequence == null) || (seed == null)) {
                    error(535);
                    return;
                    }

               //---------------------------------------------------------------
               //  build the challenge
               //---------------------------------------------------------------
               StringBuffer challenge = new StringBuffer();
               challenge.append(mechanism);
               challenge.append("-");
               challenge.append(algorithm);
               challenge.append(" ");
               challenge.append(sequence);
               challenge.append(" ");
               challenge.append(seed);
               challenge.append(" ext");

               //---------------------------------------------------------------
               //  send the response
               //---------------------------------------------------------------
               StringBuffer payload = new StringBuffer();
               payload.append("<blob>");
               payload.append(InvisibleBase64.encode(challenge.toString()));
               payload.append("</blob>");

               start(payload.toString());

               //---------------------------------------------------------------
               //  save key as channel parameter
               //---------------------------------------------------------------
               String key = getUser(authenticator,"key");

               setOnChannelZero("authenticator",authenticator);
               setOnStartChannel("authenticator",authenticator);
               setOnStartChannel("saslotpkey",key);

               log("authenticate "+authenticator);
               return;
               }
  
          //--------------------------------------------------------------------
          //  see if challenge response
          //--------------------------------------------------------------------
          if(request.isChallengeResponse()) {
               //---------------------------------------------------------------
               //  get the challenge response format
               //---------------------------------------------------------------
               String response = InvisibleBase64.decode(request.getChallengeResponseString());

               //---------------------------------------------------------------
               //  build the key
               //---------------------------------------------------------------
               InvisibleOtp otp = new InvisibleOtp();

               otp.set(response);

               String hexResponse = otp.get("hex").substring(4);

               //---------------------------------------------------------------
               //  get previous key
               //---------------------------------------------------------------
               String key = get("saslotpkey");

               //---------------------------------------------------------------
               //  see if keys match
               //---------------------------------------------------------------
               if((key != null) && (key.equalsIgnoreCase(hexResponse))) {
                    //----------------------------------------------------------
                    //  success
                    //----------------------------------------------------------
                    respond("<blob status='complete' />");
                    log("logon successful");
                    return;
                    }
               else {
                    //----------------------------------------------------------
                    //  error
                    //----------------------------------------------------------
                    error(535);
                    return;
                    }
               }

          //--------------------------------------------------------------------
          //  unknown profile usage
          //--------------------------------------------------------------------
          error(550);
          }
     catch(SpaceTimeout e) {
          //--------------------------------------------------------------------
          //  timeout
          //--------------------------------------------------------------------
          fatal(e);
          } 
     catch(SpaceException e) {
          //--------------------------------------------------------------------
          //  exception
          //--------------------------------------------------------------------
          fatal(e);
          }
     catch(InvisibleException e) {
          //--------------------------------------------------------------------
          //  otp exception
          //--------------------------------------------------------------------
          fatal(e);
          }
     }
}
