//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceModuleThread extends Thread implements SpaceModule {

protected SpaceConfig                   config;
protected SpaceUserCache                userCache;
protected SpaceProfile                  profile;
protected SpaceServer                   server;
protected SpaceRequest                  request;

//------------------------------------------------------------------------------
//  run
//------------------------------------------------------------------------------
public void run() {
     }

//------------------------------------------------------------------------------
//  start channel with payload
//------------------------------------------------------------------------------
public void start(String payload) throws SpaceTimeout,SpaceException {
     server.start(request.getStartChannel(),request.getStartProfile(),request.getSerial(),payload); 
     }

//------------------------------------------------------------------------------
//  respond
//------------------------------------------------------------------------------
public void respond(String payload) throws SpaceTimeout,SpaceException {
     server.rsp(request.getChannelElement(),payload,request.getSerial());
     }

//------------------------------------------------------------------------------
//  error
//------------------------------------------------------------------------------
public void error(int messageCode) throws SpaceTimeout,SpaceException {
     server.error(request.getChannelElement(),messageCode,request.getSerial());
     server.log(SpaceLog.error,messageCode,request);
     }

//------------------------------------------------------------------------------
//  error
//------------------------------------------------------------------------------
public void error(int messageCode,String errorString) throws SpaceTimeout,SpaceException {
     server.error(request.getChannelElement(),messageCode,errorString,request.getSerial());
     server.log(SpaceLog.error,errorString,request);
     }

//------------------------------------------------------------------------------
//  fatal
//------------------------------------------------------------------------------
public void fatal(Exception e) {
     try {
          server.log(SpaceLog.fatal,e.toString(),request);
          }
     catch(SpaceException e2) {
          }
     }

//------------------------------------------------------------------------------
//  log
//------------------------------------------------------------------------------
public void log(String message) throws SpaceException {
     server.log(SpaceLog.info,message,request);
     }

//------------------------------------------------------------------------------
//  log
//------------------------------------------------------------------------------
public void log(int level,String message) throws SpaceException {
     server.log(level,message,request);
     }

//------------------------------------------------------------------------------
//  channel parameters
//------------------------------------------------------------------------------
public void set(String key,String value) {
     request.getChannelElement().setParameter(key,value);
     }

public String get(String key) {
     return(request.getChannelElement().getParameter(key));
     }

//------------------------------------------------------------------------------
//  channel parameters on channel designated in start request
//------------------------------------------------------------------------------
public void setOnChannelZero(String key,String value) throws SpaceException {
     set(0,key,value);
     }

public String getFromChannelZero(String key) throws SpaceException {
     return(get(0,key));
     }

//------------------------------------------------------------------------------
//  channel parameters on channel designated in start request
//------------------------------------------------------------------------------
public void setOnStartChannel(String key,String value) throws SpaceException {
     set(request.getStartChannel(),key,value);
     }

public String getFromStartChannel(String key) throws SpaceException {
     return(get(request.getStartChannel(),key));
     }

//------------------------------------------------------------------------------
//  channel parameters on specified channel
//------------------------------------------------------------------------------
private void set(int channel,String key,String value) throws SpaceException {
     SpaceChannel channelElement = server.getChannel(channel);
     if(channelElement == null) {
          throw new SpaceException("SpaceModuleThread: Channel Not Open: " + channel);
          }
     channelElement.setParameter(key,value);
     }

private String get(int channel,String key) throws SpaceException {
     SpaceChannel channelElement = server.getChannel(channel);
     if(channelElement == null) {
          throw new SpaceException("SpaceModuleThread: Channel Not Open: " + channel);
          }
     return(channelElement.getParameter(key));
     }

//------------------------------------------------------------------------------
//  load users (if empty)
//------------------------------------------------------------------------------
public void loadUser() throws SpaceException {
     if(userCache.size() == 0) {
          userCache.load(config,profile);
          }
     }

//------------------------------------------------------------------------------
//  load user item
//------------------------------------------------------------------------------
public String getUser(String authenticator,String name) {
     return(userCache.get(authenticator,"authentication",name));
     }

//------------------------------------------------------------------------------
//  common request access
//------------------------------------------------------------------------------
public boolean isStart() throws SpaceException { return(request.isStart()); }
public SpaceChannel getChannel() { return(request.getChannelElement()); }
public int getSerial() { return(request.getSerial()); }
 
//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public SpaceConfig getConfig() { return(config); }
public SpaceUserCache getUserCache() { return(userCache); }

public void setConfig(SpaceConfig value) { config = value; }
public void setUserCache(SpaceUserCache value) { userCache = value; }
public void setProfile(SpaceProfile value) { profile = value; }
public void setServer(SpaceServer value) { server = value; }
public void setRequest(SpaceRequest value) { request = value; }
}
