//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Date;

public class InvisibleDate extends GregorianCalendar {

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleDate() {
     super();
     }

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleDate(int year,int month,int date,int hour,int minute,int second) {
     super(year,month,date,hour,minute,second);
     }

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleDate(long epoch) {
     super();
     setTime(new Date(epoch));
     }

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleDate(String datestring) throws InvisibleDateException {
     //-------------------------------------------------------------------------
     //  default is now
     //-------------------------------------------------------------------------
     super();

     if((datestring == null) || (datestring.length() == 0)) return;

     //-------------------------------------------------------------------------
     //  parse the string generously
     //-------------------------------------------------------------------------
     InvisibleVector v = InvisibleUtil.parse(datestring,"/.: -,");

     int size = v.size();

     //-------------------------------------------------------------------------
     //  must be at least month/day/year
     //-------------------------------------------------------------------------
     if(size < 3) {
          throw new InvisibleDateException("InvisibleDate: Invalid Date Format: " + datestring);
          }

     //-------------------------------------------------------------------------
     //  see if constructing from email Date: or Received: header element
     //-------------------------------------------------------------------------
     String email = v.getString(0);

     if((email.equalsIgnoreCase("date")) || (email.equalsIgnoreCase("received"))) {
          setFromEmailDate(v);
          return;
          }

     //-------------------------------------------------------------------------
     //  see if yyyy mm dd or mm dd yyyy format
     //-------------------------------------------------------------------------
     String year = null;
     String month = null;
     String day = null;

     int length0 = v.getString(0).length();
     int length2 = v.getString(2).length();

     if((length0 == 4) && (length2 != 4)) {
          //--------------------------------------------------------------------
          //  yyyy mm dd format
          //--------------------------------------------------------------------
          year = v.getString(0);
          month = v.getString(1);
          day = v.getString(2);
          }
     else {
          if(length2 != 4) {
               //---------------------------------------------------------------
               //  year must be entered as a 4 digit number
               //---------------------------------------------------------------
               throw new InvisibleDateException("InvisibleDate: Invalid Date Format: " + datestring);
               }
          else {
               //---------------------------------------------------------------
               //  mm dd yyyy format
               //---------------------------------------------------------------
               month = v.getString(0);
               day = v.getString(1);
               year = v.getString(2);
               }
          }

     //-------------------------------------------------------------------------
     //  convert text-based month if appropriate
     //-------------------------------------------------------------------------
     int yearInt;
     int monthInt;
     int dayInt;

     try {
          yearInt = Integer.parseInt(year);
          if((monthInt = month(month)) == -1) {
               monthInt = Integer.parseInt(month);
               }
          else {
               ++monthInt;
               }
          dayInt = Integer.parseInt(day);
          }
     catch(NumberFormatException e) {
          throw new InvisibleDateException("InvisibleDate: Invalid Date Format: " + datestring);
          }

     //-------------------------------------------------------------------------
     //  get hh/mm/ss if appropriate
     //-------------------------------------------------------------------------
     int hourInt = 0;
     int minuteInt = 0;
     int secondInt = 0;

     try {
          if(size >= 4) {
               hourInt = Integer.parseInt(v.getString(3));

               if(size >= 5) {
                    minuteInt = Integer.parseInt(v.getString(4));

                    if(size >= 6) {
                         secondInt = Integer.parseInt(v.getString(5));
                         }
                    }
               }
          }
     catch(NumberFormatException e) {
          throw new InvisibleDateException("InvisibleDate: Invalid Time Format: " + datestring);
          }

     //-------------------------------------------------------------------------
     //  build new date
     //-------------------------------------------------------------------------
     set(yearInt,--monthInt,dayInt,hourInt,minuteInt,secondInt);
     }

//------------------------------------------------------------------------------
//  set from email header
//------------------------------------------------------------------------------
private void setFromEmailDate(InvisibleVector v)  throws InvisibleDateException {
     //-------------------------------------------------------------------------
     //  first item of vector is email or received
     //-------------------------------------------------------------------------
     int year = 0;
     int month = 0;
     int day = 0;
     int hour = 0;
     int minute = 0;
     int second = 0;
     String ampm = null;
 
     //-------------------------------------------------------------------------
     //  format 12/14/99 10:52 AM
     //-------------------------------------------------------------------------
     if(v.size() == 7) {
          month = InvisibleUtil.parseInt((String)(v.elementAt(1)));
          day = InvisibleUtil.parseInt((String)(v.elementAt(2)));
          year = InvisibleUtil.parseInt((String)(v.elementAt(3)));
          hour = InvisibleUtil.parseInt((String)(v.elementAt(4)));
          minute = InvisibleUtil.parseInt((String)(v.elementAt(5)));
          ampm = (String)(v.elementAt(6));
          }

     //-------------------------------------------------------------------------
     //  format 01/03  8:54 AM (current year implied)
     //-------------------------------------------------------------------------
     if(v.size() == 6) {
          month = InvisibleUtil.parseInt((String)(v.elementAt(1)));
          day = InvisibleUtil.parseInt((String)(v.elementAt(2)));
          InvisibleDate getyear = new InvisibleDate();
          year = getyear.getYear();
          hour = InvisibleUtil.parseInt((String)(v.elementAt(3)));
          minute = InvisibleUtil.parseInt((String)(v.elementAt(4)));
          ampm = (String)(v.elementAt(5));
          }
 
     //-------------------------------------------------------------------------
     //  convert time to military time
     //-------------------------------------------------------------------------
     if(ampm.equalsIgnoreCase("pm")) hour += 12;
 
     //-------------------------------------------------------------------------
     //  y2k handling
     //-------------------------------------------------------------------------
     if((year >= 90) && (year <= 99)) year += 1900;
     if(year < 90) year += 2000;

     //-------------------------------------------------------------------------
     //  build new date
     //-------------------------------------------------------------------------
     set(year,--month,day,hour,minute,second);
     }

//------------------------------------------------------------------------------
//  toString
//------------------------------------------------------------------------------
public String toString() {
     return(getDateString());
     }

//------------------------------------------------------------------------------
//  get standard date string
//------------------------------------------------------------------------------
public String getDateString() {
     StringBuffer sb = new StringBuffer();
     sb.append(getYear());
     sb.append("-");
     sb.append(InvisibleUtil.pad(getMonthOfYear(),2,'0'));
     sb.append("-");
     sb.append(InvisibleUtil.pad(getDay(),2,'0'));
     sb.append("-");
     sb.append(InvisibleUtil.pad(getHourOfDay(),2,'0'));
     sb.append("-");
     sb.append(InvisibleUtil.pad(getMinute(),2,'0'));
     sb.append("-");
     sb.append(InvisibleUtil.pad(getSecond(),2,'0'));

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  get standard log date/time string
//------------------------------------------------------------------------------
public String getLogDateString() {
     StringBuffer sb = new StringBuffer();
     sb.append(InvisibleUtil.pad(String.valueOf(getMonthOfYear()),2,'0'));
     sb.append("/");
     sb.append(InvisibleUtil.pad(String.valueOf(getDay()),2,'0'));
     sb.append(" ");
     sb.append(InvisibleUtil.pad(String.valueOf(getHourOfDay()),2,'0'));
     sb.append(":");
     sb.append(InvisibleUtil.pad(String.valueOf(getMinute()),2,'0'));
     sb.append(":");
     sb.append(InvisibleUtil.pad(String.valueOf(getSecond()),2,'0'));
     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  month conversion
//------------------------------------------------------------------------------
public static String[] monthShort = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
public static String[] monthLong = {"January","February","March","April","May","June","July","August","September","October","November","December"};

public int month(String s) {
     for(int k=0;k<12;++k) {
          if(s.equalsIgnoreCase(monthShort[k])) return(k);
          if(s.equalsIgnoreCase(monthLong[k])) return(k);
          }

     return(-1);
     }

//------------------------------------------------------------------------------
//  year short (last two digits)
//------------------------------------------------------------------------------
public String getYearShort() {
     return((String.valueOf(getYear())).substring(2,4));
     }

//------------------------------------------------------------------------------
//  get specified element
//------------------------------------------------------------------------------
public String get(String modifier) {
     String modifierLc = modifier.toLowerCase();

     try {
          if(modifierLc.equals("datestring")) return(getDateString());
          if(modifierLc.equals("string")) return(getDateString());
          if(modifierLc.equals("default")) return(getDateString());
          if(modifierLc.equals("year")) return(String.valueOf(getYear()));
          if(modifierLc.equals("yearshort")) return(getYearShort());
          if(modifierLc.equals("day")) return(String.valueOf(getDay()));
          if(modifierLc.equals("monthofyear")) return(String.valueOf(getMonthOfYear()));
          if(modifierLc.equals("monthtext")) return(String.valueOf(getMonthTextLong()));
          if(modifierLc.equals("monthtextlong")) return(String.valueOf(getMonthTextLong()));
          if(modifierLc.equals("monthtextshort")) return(String.valueOf(getMonthTextShort()));
          if(modifierLc.equals("era")) return(String.valueOf(getEra()));
          if(modifierLc.equals("month")) return(String.valueOf(getMonth()));
          if(modifierLc.equals("weekofyear")) return(String.valueOf(getWeekOfYear()));
          if(modifierLc.equals("weekofmonth")) return(String.valueOf(getWeekOfMonth()));
          if(modifierLc.equals("date")) return(String.valueOf(getDate()));
          if(modifierLc.equals("dayofmonth")) return(String.valueOf(getDayOfMonth()));
          if(modifierLc.equals("dayofyear")) return(String.valueOf(getDayOfYear()));
          if(modifierLc.equals("dayofweek")) return(String.valueOf(getDayOfWeek()));
          if(modifierLc.equals("dayofweekinmonth")) return(String.valueOf(getDayOfWeekInMonth()));
          if(modifierLc.equals("ampm")) return(String.valueOf(getAmPm()));
          if(modifierLc.equals("ampmtext")) return(String.valueOf(getAmPmText()));
          if(modifierLc.equals("hour")) return(String.valueOf(getHour()));
          if(modifierLc.equals("hourofday")) return(String.valueOf(getHourOfDay()));
          if(modifierLc.equals("minute")) return(String.valueOf(getMinute()));
          if(modifierLc.equals("second")) return(String.valueOf(getSecond()));
          if(modifierLc.equals("millisecond")) return(String.valueOf(getMillisecond()));
          if(modifierLc.equals("zoneoffset")) return(String.valueOf(getZoneOffset()));
          if(modifierLc.equals("dstoffset")) return(String.valueOf(getDstOffset()));
          if(modifierLc.equals("time")) {
               StringBuffer sb = new StringBuffer();
               sb.append(String.valueOf(getHour()));
               sb.append(":");
               sb.append(String.valueOf(getMinute()));
               sb.append(":");
               sb.append(String.valueOf(getSecond()));
               sb.append(" ");
               sb.append(String.valueOf(getAmPmText()));
               return(sb.toString());
               }
          if((modifierLc.equals("standard")) || (modifierLc.equals("monthdayyearlong"))) {
               StringBuffer sb = new StringBuffer();
               sb.append(String.valueOf(getMonthTextLong()));
               sb.append(" ");
               sb.append(String.valueOf(getDay()));
               sb.append(", ");
               sb.append(String.valueOf(getYear()));
               return(sb.toString());
               }
          if((modifierLc.equals("standardshort")) || (modifierLc.equals("monthdayyearshort"))) {
               StringBuffer sb = new StringBuffer();
               sb.append(InvisibleUtil.pad(String.valueOf(getMonthOfYear()),2,'0'));
               sb.append(".");
               sb.append(InvisibleUtil.pad(String.valueOf(getDay()),2,'0'));
               sb.append(".");
               sb.append(String.valueOf(getYearShort()));
               return(sb.toString());
               }
          if(modifierLc.equals("monthdayyear")) {
               StringBuffer sb = new StringBuffer();
               sb.append(String.valueOf(getMonthOfYear()));
               sb.append("/");
               sb.append(String.valueOf(getDay()));
               sb.append("/");
               sb.append(String.valueOf(getYear()));
               return(sb.toString());
               }
          if(modifierLc.equals("yearmonthday")) {
               StringBuffer sb = new StringBuffer();
               sb.append(String.valueOf(getYear()));
               sb.append("-");
               sb.append(String.valueOf(getMonthOfYear()));
               sb.append("-");
               sb.append(String.valueOf(getDay()));
               return(sb.toString());
               }

          //--------------------------------------------------------------------
          //  date math
          //--------------------------------------------------------------------
          int math = 0;
          int multiplier = 0;

          if(modifierLc.startsWith("plus")) {
               math = 1;
               modifierLc = modifierLc.substring(4);
               }

          if(modifierLc.startsWith("minus")) {
               math = -1;
               modifierLc = modifierLc.substring(5);
               }

          if(math != 0) {
               if(modifierLc.endsWith("seconds")) {
                    multiplier = 1;
                    modifierLc = modifierLc.substring(0,modifierLc.length()-7);
                    }
               else {
                    if(modifierLc.endsWith("minutes")) {
                         multiplier = 60;
                         modifierLc = modifierLc.substring(0,modifierLc.length()-7);
                         }
                    else {
                         if(modifierLc.endsWith("hours")) {
                              multiplier = 60 * 60;
                              modifierLc = modifierLc.substring(0,modifierLc.length()-5);
                              }
                         else {
                              if(modifierLc.endsWith("days")) {
                                   multiplier = 60 * 60 * 24;
                                   modifierLc = modifierLc.substring(0,modifierLc.length()-4);
                                   }
                              }
                         }
                    }

               add(Calendar.SECOND,InvisibleUtil.parseInt(modifierLc) * multiplier * math);

               return(toString());
               }
          }
     catch(NumberFormatException e) {
          }

     return(null);
     }

//------------------------------------------------------------------------------
//  simple date access
//------------------------------------------------------------------------------
public int getDay() { return(get(Calendar.DAY_OF_MONTH)); }
public int getMonthOfYear() { return(get(Calendar.MONTH)+1); }

//------------------------------------------------------------------------------
//  textual month access
//------------------------------------------------------------------------------
public String getMonthTextShort() { return(monthShort[getMonth()]); }
public String getMonthTextLong() { return(monthLong[getMonth()]); }

//------------------------------------------------------------------------------
//  textual am/pm access
//------------------------------------------------------------------------------
public String getAmPmText() {
     int ampm = getAmPm();
     if(ampm == Calendar.AM) return("AM");
     if(ampm == Calendar.PM) return("PM");
     return("");
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public int getEra() { return(get(Calendar.ERA)); }
public int getYear() { return(get(Calendar.YEAR)); }
public int getMonth() { return(get(Calendar.MONTH)); }
public int getWeekOfYear() { return(get(Calendar.WEEK_OF_YEAR)); }
public int getWeekOfMonth() { return(get(Calendar.WEEK_OF_MONTH)); }
public int getDate() { return(get(Calendar.DATE)); }
public int getDayOfMonth() { return(get(Calendar.DAY_OF_MONTH)); }
public int getDayOfYear() { return(get(Calendar.DAY_OF_YEAR)); }
public int getDayOfWeek() { return(get(Calendar.DAY_OF_WEEK)); }
public int getDayOfWeekInMonth() { return(get(Calendar.DAY_OF_WEEK_IN_MONTH)); }
public int getAmPm() { return(get(Calendar.AM_PM)); }
public int getHour() { return(get(Calendar.HOUR)); }
public int getHourOfDay() { return(get(Calendar.HOUR_OF_DAY)); }
public int getMinute() { return(get(Calendar.MINUTE)); }
public int getSecond() { return(get(Calendar.SECOND)); }
public int getMillisecond() { return(get(Calendar.MILLISECOND)); }
public int getZoneOffset() { return(get(Calendar.ZONE_OFFSET)/(60*60*1000)); }
public int getDstOffset() { return(get(Calendar.DST_OFFSET)/(60*60*1000)); }
}
