//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;
import java.util.Properties;
import java.util.Date;
import java.util.Enumeration;
import java.io.InputStream;
import java.io.IOException;

class InvisibleEmail {

String                            mailServer;
String                            mailProtocol;
String                            mailUsername;
String                            mailPassword;
String                            mailFolder;

Store                             store;
Folder                            folder;
Message                           message;
int                               messageNumber;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleEmail() {
     store = null;
     folder = null;
     message = null;
     messageNumber = 1;

     mailProtocol = "pop3";
     mailFolder = "INBOX";
     }

//------------------------------------------------------------------------------
//  connect
//------------------------------------------------------------------------------
public void connect() throws InvisibleException {
     try {
          //--------------------------------------------------------------------
          //  get the session object
          //--------------------------------------------------------------------
          Properties props = System.getProperties();
          Session session = Session.getDefaultInstance(props,null);

          //--------------------------------------------------------------------
          //  get the store object
          //--------------------------------------------------------------------
          store = session.getStore(mailProtocol);
          store.connect(mailServer,mailUsername,mailPassword);

          //--------------------------------------------------------------------
          //  get the folder object
          //--------------------------------------------------------------------
          folder = store.getDefaultFolder();

          if(folder == null) {
               throw new InvisibleException("InvisibleEmail: No Default Folder");
               }

          folder = folder.getFolder(mailFolder);

          if(folder == null) {
               throw new InvisibleException("InvisibleEmail: No Folder");
               }

          //--------------------------------------------------------------------
          //  open the folder
          //--------------------------------------------------------------------
          folder.open(Folder.READ_WRITE);
          }
     catch(NoSuchProviderException e) {
          throw new InvisibleException("InvisibleEmail: " + e.toString());
          }
     catch(MessagingException e) {
          throw new InvisibleException("InvisibleEmail: " + e.toString());
          }
     }

//------------------------------------------------------------------------------
//  get message
//------------------------------------------------------------------------------
public InvisibleEmailMessage getMessage() {
     try {
          //--------------------------------------------------------------------
          //  get the next message
          //--------------------------------------------------------------------
          message = null;

          if(folder == null) return(null);

          message = (MimeMessage)(folder.getMessage(messageNumber++));
          if(message == null) return(null);

          //--------------------------------------------------------------------
          //  instantiate a new message
          //--------------------------------------------------------------------
          InvisibleEmailMessage emailMessage = new InvisibleEmailMessage();

          //--------------------------------------------------------------------
          //  set the subject
          //--------------------------------------------------------------------
          emailMessage.setSubject(message.getSubject());

          //--------------------------------------------------------------------
          //  set from
          //--------------------------------------------------------------------
          Address[] addresses;

          addresses = message.getFrom();
          if((addresses != null) && (addresses.length > 0)) {
               emailMessage.setFrom(addresses[0].toString());
               }

          //--------------------------------------------------------------------
          //  set bcc
          //--------------------------------------------------------------------
          addresses = message.getRecipients(Message.RecipientType.BCC);
          if((addresses != null) && (addresses.length > 0)) {
               for(int k=0;k<addresses.length;++k) {
                    emailMessage.setBcc(addresses[k].toString());
                    }
               }

          //--------------------------------------------------------------------
          //  set cc
          //--------------------------------------------------------------------
          addresses = message.getRecipients(Message.RecipientType.CC);
          if((addresses != null) && (addresses.length > 0)) {
               for(int k=0;k<addresses.length;++k) {
                    emailMessage.setCc(addresses[k].toString());
                    }
               }

          //--------------------------------------------------------------------
          //  set to
          //--------------------------------------------------------------------
          addresses = message.getRecipients(Message.RecipientType.TO);
          if((addresses != null) && (addresses.length > 0)) {
               for(int k=0;k<addresses.length;++k) {
                    emailMessage.setTo(addresses[k].toString());
                    }
               }

          //--------------------------------------------------------------------
          //  set date sent
          //--------------------------------------------------------------------
          emailMessage.setDateSent(message.getSentDate());

          //--------------------------------------------------------------------
          //  set date received
          //--------------------------------------------------------------------
          emailMessage.setDateReceived(message.getReceivedDate());

          //--------------------------------------------------------------------
          //  get message content
          //--------------------------------------------------------------------
          Object o = message.getContent();

          //--------------------------------------------------------------------
          //  see if string
          //--------------------------------------------------------------------
          if(o instanceof String) {
               InvisibleEmailContent content = new InvisibleEmailContent();
               content.setContentType(message.getContentType());
               content.setContent((String)(o));
               emailMessage.setContent(content);
               }

          //--------------------------------------------------------------------
          //  see if multipart
          //--------------------------------------------------------------------
          if(o instanceof Multipart) {
               Multipart mp = (Multipart)(o);

               int size = mp.getCount();

               for(int k=0;k<size;++k) {
                    //----------------------------------------------------------
                    //  instantiate new content
                    //----------------------------------------------------------
                    InvisibleEmailContent content = new InvisibleEmailContent();

                    //----------------------------------------------------------
                    //  get each part
                    //----------------------------------------------------------
                    MimeBodyPart bodypart = (MimeBodyPart)(mp.getBodyPart(k));

                    o = bodypart.getContent();

                    //----------------------------------------------------------
                    //  set the content type
                    //----------------------------------------------------------
                    content.setContentType(bodypart.getContentType());

                    //----------------------------------------------------------
                    //  part is a string
                    //----------------------------------------------------------
                    if(o instanceof String) {
                         content.setContent((String)(o));
                         }

                    //----------------------------------------------------------
                    //  part is an encoded stream
                    //----------------------------------------------------------
                    if(o instanceof InputStream) {
                         StringBuffer sb = new StringBuffer();

                         InputStream input = (InputStream)(o);
                         int c;
                         while((c = input.read()) != -1) sb.append((char)(c & 0xff));
                         input.close();

                         content.setContent(sb.toString());
                         }

                    //----------------------------------------------------------
                    //  save the content element in the message
                    //----------------------------------------------------------
                    emailMessage.setContent(content);
                    }
               }

          return(emailMessage);
          }
     catch(ArrayIndexOutOfBoundsException e) {
          return(null);
          }
     catch(MessagingException e) {
          return(null);
          }
     catch(IOException e) {
          return(null);
          }
     }

//------------------------------------------------------------------------------
//  remove message
//------------------------------------------------------------------------------
public void removeMessage() {
     try {
          if(message != null) message.setFlag(Flags.Flag.DELETED,true);
          //if(folder != null) folder.expunge();
          }
     catch(MessagingException e) {
          }
     }
     
//------------------------------------------------------------------------------
//  close
//------------------------------------------------------------------------------
public void close() {
     try {
          if(folder != null) folder.close(true);
          if(store != null) store.close();
          }
     catch(MessagingException e) {
          }
     finally {
          folder = null;
          store = null;
          }
     }

//------------------------------------------------------------------------------
//  send specified message
//------------------------------------------------------------------------------
public boolean send(InvisibleEmailMessage message) throws InvisibleException {
     //-------------------------------------------------------------------------
     //  not an error if no message
     //-------------------------------------------------------------------------
     if(message == null) return(true);

     //-------------------------------------------------------------------------
     //  get message items
     //-------------------------------------------------------------------------
     String from = message.getFrom();
     String subject = message.getSubject();
     if(message.getContentSize() <= 0) {
          throw new InvisibleException("InvisibleEmail: No Content");
          }
     String content = message.getContent(0).getContent();
     InvisibleVector toVector = message.getTo();

     //-------------------------------------------------------------------------
     //  not an error if no one on the list for sending
     //-------------------------------------------------------------------------
     if((toVector == null) || (toVector.size() == 0)) return(true);

     //-------------------------------------------------------------------------
     //  all settings must be made to even try sending the message
     //-------------------------------------------------------------------------
     if(mailServer == null) return(false);
     if(from == null) return(false);
     if(subject == null) return(false);
     //if(toVector == null) return(false);
     if(content == null) return(false);

     //-------------------------------------------------------------------------
     //  set mail server and instantiate the session
     //-------------------------------------------------------------------------
     Properties properties = new Properties();
     properties.put("mail.smtp.host",getMailServer());

     Session session = Session.getDefaultInstance(properties,null);

     try {
          //--------------------------------------------------------------------
          //  build the required "to" array
          //--------------------------------------------------------------------
          int size = toVector.size();

          InternetAddress[] address = new InternetAddress[size];

          for(int k=0;k<size;++k) address[k] = new InternetAddress(toVector.getString(k));

          //--------------------------------------------------------------------
          //  send the message
          //--------------------------------------------------------------------
          Message msg = new MimeMessage(session);
          msg.setFrom(new InternetAddress(from));
          msg.setRecipients(Message.RecipientType.TO, address);
          msg.setSubject(subject);
          msg.setSentDate(new Date());
          msg.setText(content);
        
          Transport.send(msg);
          }
     //catch(javax.mail.internet.AddressException e) {
     catch(AddressException e) {
          throw new InvisibleException("InvisibleEmailSend: " + e.toString());
          }
     catch(SendFailedException e) {
          throw new InvisibleException("InvisibleEmailSend: " + e.toString());
          }
     catch(MessagingException e) {
          throw new InvisibleException("InvisibleEmailSend: " + e.toString());
          }

     return(true);
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public void setMailServer(String value) { mailServer = value; }
public void setMailProtocol(String value) { mailProtocol = value; }
public void setMailUsername(String value) { mailUsername = value; }
public void setMailPassword(String value) { mailPassword = value; }
public void setMailFolder(String value) { mailFolder = value; }

public String getMailServer() { return(mailServer); }
public String getMailProtocol() { return(mailProtocol); }
public String getMailUsername() { return(mailUsername); }
public String getMailPassword() { return(mailPassword); }
public String getMailFolder() { return(mailFolder); }
}
