//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.StringBufferInputStream;
import java.io.IOException;
import java.util.Date;

public class InvisibleEmailMessage {

InvisibleVector                          to;
String                                   toTemplate;
InvisibleVector                          cc;
String                                   ccTemplate;
InvisibleVector                          bcc;
String                                   bccTemplate;
String                                   from;
String                                   fromAddress;
String                                   fromUsername;
String                                   fromTemplate;
String                                   subject;
String                                   subjectTemplate;
String                                   subjectNoTemplate;
InvisibleDate                            dateSent;
InvisibleDate                            dateReceived;
InvisibleVector                          content;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleEmailMessage() {
     to = new InvisibleVector();
     cc = new InvisibleVector();
     bcc = new InvisibleVector();
     content = new InvisibleVector();

     toTemplate = null;
     ccTemplate = null;
     bccTemplate = null;
     }

//------------------------------------------------------------------------------
//  parse subject
//------------------------------------------------------------------------------
public void parseSubject(String subject) {
     if(subject == null) return;
     int index = subject.indexOf(":");
     if(index != -1) {
          try {
               subjectTemplate = subject.substring(0,index).trim();
               subjectNoTemplate = subject.substring(++index).trim();
               }
          catch(StringIndexOutOfBoundsException e) {
               subjectNoTemplate = "";
               }
          }
     else {
          subjectTemplate = "";
          subjectNoTemplate = subject;
          }
     }

//------------------------------------------------------------------------------
//  parse from
//------------------------------------------------------------------------------
public void parseFrom(String from) {
     if(from == null) return;
     InvisibleVector v = InvisibleUtil.parse(from,"<>\"' \t\n\r:!$^&*()/,");

     int size = v.size();

     for(int k=0;k<v.size();++k) {
          String fromTemp = v.getString(k);
          int index;
          if((index = fromTemp.indexOf("@")) != -1) {
               fromAddress = fromTemp;
               fromUsername = fromAddress.substring(0,index);

               if((index = fromUsername.indexOf("+")) != -1) {
                    try {
                         fromTemplate = fromUsername.substring(index+1);
                         fromUsername = fromUsername.substring(0,index);
                         }
                    catch(StringIndexOutOfBoundsException e) {
                         fromTemplate = "";
                         }
                    }
               else {
                    fromTemplate = "";
                    }

               break;
               }
          }
     }

//------------------------------------------------------------------------------
//  parse email address for template
//------------------------------------------------------------------------------
public String parseAddress(String address) {
     if(address == null) return(null);
     int indexend;
     if((indexend = address.indexOf("@")) != -1) {
          int index;
          if(((index = address.indexOf("+")) != -1) && (index < indexend)) {
               try {
                    return(address.substring(++index,indexend));
                    }
               catch(StringIndexOutOfBoundsException e) {
                    return(null);
                    }
               }
          }

     return(null);
     }

//------------------------------------------------------------------------------
//  extract content from an email header element
//------------------------------------------------------------------------------
static private String extract(String s) {
     if(s == null) return(null);
     try {
          int index = s.indexOf(":");
          if(index == -1) return(s);
          return(s.substring(++index).trim());
          }
     catch(StringIndexOutOfBoundsException e) {
          return("");
          }
     }

//------------------------------------------------------------------------------
//  to vector access methods
//------------------------------------------------------------------------------
public void setTo(String value) {
     to.set(value);
     String s;
     if((s = parseAddress(value)) != null) toTemplate = s;
     }

public String getTo(int item) {
     return((String)(to.get(item)));
     }

public int getToSize() {
     return(to.size());
     }

//------------------------------------------------------------------------------
//  cc vector access methods
//------------------------------------------------------------------------------
public void setCc(String value) {
     cc.set(value);
     String s;
     if((s = parseAddress(value)) != null) ccTemplate = s;
     }

public String getCc(int item) {
     return((String)(cc.get(item)));
     }

public int getCcSize() {
     return(cc.size());
     }

//------------------------------------------------------------------------------
//  bcc vector access methods
//------------------------------------------------------------------------------
public void setBcc(String value) {
     bcc.set(value);
     String s;
     if((s = parseAddress(value)) != null) bccTemplate = s;
     }

public String getBcc(int item) {
     return((String)(bcc.get(item)));
     }

public int getBccSize() {
     return(bcc.size());
     }

//------------------------------------------------------------------------------
//  set content vector item for plain text message
//------------------------------------------------------------------------------
public void setContent(String s) {
     InvisibleEmailContent content = new InvisibleEmailContent();
     content.setMime("text/plain");
     content.setContent(s);
     setContent(content);
     }

//------------------------------------------------------------------------------
//  content vector access methods
//------------------------------------------------------------------------------
public void setContent(InvisibleEmailContent contentElement) {
     content.add(contentElement);
     }

public InvisibleEmailContent getContent(int item) {
     return((InvisibleEmailContent)(content.get(item)));
     }

public int getContentSize() {
     return(content.size());
     }

public InvisibleEmailContent getContentWithNoName() {
     int size = getContentSize();
     for(int k=0;k<size;++k) {
          InvisibleEmailContent content = getContent(k);
          if(content.getName() == null) return(content);
          }
     return(null);
     }

//------------------------------------------------------------------------------
//  set from item from name and address
//------------------------------------------------------------------------------
public void setFrom(String name,String address) {
     if(name == null) {
          setFrom(address);
          } 
     else { 
          if(address == null) {
               setFrom(name);
               }
          else {
               setFrom(name+" <"+address+">");
               }
          }
     }

//------------------------------------------------------------------------------
//  set and parse from
//------------------------------------------------------------------------------
public void setFrom(String value) {
     from = value;
     parseFrom(from);
     }

//------------------------------------------------------------------------------
//  set and parse subject
//------------------------------------------------------------------------------
public void setSubject(String value) {
     subject = value;
     parseSubject(subject);
     }

//------------------------------------------------------------------------------
//  set date
//------------------------------------------------------------------------------
public InvisibleDate setDateSent(Date d) {
     if(d == null) return(null);
     dateSent = new InvisibleDate(d.getYear()+1900,d.getMonth(),d.getDate(),d.getHours(),d.getMinutes(),d.getSeconds());
     return(dateSent);
     }

//------------------------------------------------------------------------------
//  set received date
//------------------------------------------------------------------------------
public InvisibleDate setDateReceived(Date d) {
     if(d == null) return(null);
     dateReceived = new InvisibleDate(d.getYear()+1900,d.getMonth(),d.getDate(),d.getHours(),d.getMinutes(),d.getSeconds());
     return(dateReceived);
     }

//------------------------------------------------------------------------------
//  default template from the from address
//------------------------------------------------------------------------------
public String getTemplate() {
     if(toTemplate != null) return(toTemplate);
     if(ccTemplate != null) return(ccTemplate);
     return(bccTemplate);
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public void setTo(InvisibleVector value) { to = value; } 
public void setCc(InvisibleVector value) { cc = value; } 
public void setBcc(InvisibleVector value) { bcc = value; } 

public InvisibleVector getTo() { return(to); }
public String getToTemplate() { return(toTemplate); }
public InvisibleVector getCc() { return(cc); }
public String getCcTemplate() { return(ccTemplate); }
public InvisibleVector getBcc() { return(bcc); }
public String getBccTemplate() { return(bccTemplate); }
public String getFrom() { return(from); }
public String getFromAddress() { return(fromAddress); }
public String getFromUsername() { return(fromUsername); }
public String getFromTemplate() { return(fromTemplate); }
public String getSubject() { return(subject); }
public String getSubjectTemplate() { return(subjectTemplate); }
public String getSubjectNoTemplate() { return(subjectNoTemplate); }
public InvisibleDate getDateSent() { return(dateSent); }
public InvisibleDate getDateReceived() { return(dateReceived); }
public InvisibleVector getContent() { return(content); }
}
