//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

public class InvisibleExec {

InvisibleVector                         args;
InvisibleVector                         result;
int                                     status;
String                                  outputStreamContent;
String                                  errorStreamContent;
String                                  delim;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleExec() {
     clear();
     status = 0;
     outputStreamContent = null;
     errorStreamContent = null;
     }

//------------------------------------------------------------------------------
//  exec
//------------------------------------------------------------------------------
public void exec() throws InvisibleException {
     if((args == null) || (args.size() == 0)) return;

     String command[] = new String[args.size()];
     int size = args.size();
     for(int k=0;k<size;++k) command[k] = args.getString(k);

     //DEBUG.String s = exec(command);
     exec(command);

     //DEBUGresult = InvisibleUtil.parse(s,delim);
     }

//------------------------------------------------------------------------------
//  exec
//------------------------------------------------------------------------------
public String exec(String[] command) throws InvisibleException {
     try {
          Process p = Runtime.getRuntime().exec(command);
          p.waitFor();

          return(getStreams(p));
          }
     catch(InterruptedException e) {
          throw new InvisibleException("InvisibleExec: " + e.toString());
          }
     catch(IOException e) {
          throw new InvisibleException("InvisibleExec: " + e.toString());
          }
     }

//------------------------------------------------------------------------------
//  exec
//------------------------------------------------------------------------------
public String exec(String command) throws InvisibleException {
     try {
          Process p = Runtime.getRuntime().exec(command);
          p.waitFor();

          return(getStreams(p));
          }
     catch(InterruptedException e) {
          throw new InvisibleException("InvisibleExec: " + e.toString());
          }
     catch(IOException e) {
          throw new InvisibleException("InvisibleExec: " + e.toString());
          }
     }

//------------------------------------------------------------------------------
//  getStreams
//------------------------------------------------------------------------------
private String getStreams(Process p) throws IOException {
     status = p.exitValue();
     outputStreamContent = getOutputStream(p);
     errorStreamContent = getErrorStream(p);
     return(outputStreamContent);
     }

//------------------------------------------------------------------------------
//  getOutputStream
//------------------------------------------------------------------------------
private String getOutputStream(Process p) throws IOException {
     return(getStream(p.getInputStream()));
     }

//------------------------------------------------------------------------------
//  getErrorStream
//------------------------------------------------------------------------------
private String getErrorStream(Process p) throws IOException {
     return(getStream(p.getErrorStream()));
     }

//------------------------------------------------------------------------------
//  get specified stream
//------------------------------------------------------------------------------
private String getStream(InputStream in) throws IOException {
     InputStreamReader inReader = new InputStreamReader(in);
     BufferedReader reader = new BufferedReader(inReader);

     StringBuffer sb = new StringBuffer();
     String s;

     while((s = reader.readLine()) != null) {
          sb.append(s);
          sb.append("\n");
          }

     reader.close();
     inReader.close();

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  clear
//------------------------------------------------------------------------------
public void clear() {
     args = new InvisibleVector();
     result = null;
     delim = " ";
     }

//------------------------------------------------------------------------------
//  set argument
//------------------------------------------------------------------------------
public void setArg(String value) { args.set(value); }

//------------------------------------------------------------------------------
//  parse and result access
//------------------------------------------------------------------------------
public void parse() {
     result = InvisibleUtil.parse(outputStreamContent,delim);
     }

public String getResult(int index) {
     try {
          return(result.getString(index));
          }
     catch(ArrayIndexOutOfBoundsException e) {
          return(null);
          }
     }

public int sizeResult() { return(result.size()); }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public int getStatus() { return(status); }
public String getOutputStreamContent() { return(outputStreamContent); }
public String getErrorStreamContent() { return(errorStreamContent); }

public void setDelim(String value) { delim = value; }
}
