//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.io.File;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStreamReader;
import java.io.IOException;

public class InvisibleFileCache extends InvisibleHashtable {

boolean                                 active;
boolean                                 debug;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleFileCache() {
     super();

     //-------------------------------------------------------------------------
     //  default behavior is to not save the file content in the cache
     //-------------------------------------------------------------------------
     setActive(false);

     //-------------------------------------------------------------------------
     //  default behavior is to debug
     //-------------------------------------------------------------------------
     setDebug(true);
     }

//------------------------------------------------------------------------------
//  get if already existing
//------------------------------------------------------------------------------
public String get(String key) {
     return((String)(super.get(key)));
     }

//------------------------------------------------------------------------------
//  get with system eol
//------------------------------------------------------------------------------
public String get(String key,String pathname,boolean showFileNotFoundException) throws InvisibleException,IOException {
     return(get(key,pathname,showFileNotFoundException,null));
     }

//------------------------------------------------------------------------------
//  get with missing file reporting
//------------------------------------------------------------------------------
public String get(String key,String pathname,boolean showFileNotFoundException,String eol) throws InvisibleException,IOException {
     //-------------------------------------------------------------------------
     //  check for null key
     //-------------------------------------------------------------------------
     if(key == null) {
          if(showFileNotFoundException) {
               throw new InvisibleException("InvisibleFileCache: Null Key");
               }

          return(null);
          }

     //-------------------------------------------------------------------------
     //  see if pathname already in the cache
     //-------------------------------------------------------------------------
     String s = (String)(super.get(key));

     //-------------------------------------------------------------------------
     //  file content already in the cache
     //-------------------------------------------------------------------------
     if(s != null) {
          if(debug) System.err.println("Cached: " + key);
          return(s);
          }

     //-------------------------------------------------------------------------
     //  check for null pathname
     //-------------------------------------------------------------------------
     if(pathname == null) {
          if(showFileNotFoundException) {
               throw new InvisibleException("InvisibleFileCache: Null Pathname");
               }

          return(null);
          }

     //-------------------------------------------------------------------------
     //  load the file content
     //-------------------------------------------------------------------------
     try {
          FileInputStream f = new FileInputStream(pathname);
          BufferedReader input = new BufferedReader(new InputStreamReader(f));

          if(eol == null) eol = System.getProperty("line.separator");

          StringBuffer sb = new StringBuffer();

          while((s = input.readLine()) != null) {
               sb.append(s);
               sb.append(eol);
               }

          input.close();
          f.close();
 
          //--------------------------------------------------------------------
          //  save the content in the cache if appropriate
          //--------------------------------------------------------------------
          s = sb.toString();

          if(active) set(key,s);

          //--------------------------------------------------------------------
          //  show loading in log if debug
          //--------------------------------------------------------------------
          if(debug) System.err.println("Load: " + key + ":" + pathname);

          //--------------------------------------------------------------------
          //  return the file content
          //--------------------------------------------------------------------
          return(s);
          }
     catch(FileNotFoundException e) {
          //--------------------------------------------------------------------
          //  zero length content means file does not exist
          //--------------------------------------------------------------------
          s = new String();

          if(active) set(key,s);

          if(showFileNotFoundException) {
               throw new InvisibleException("InvisibleFileCache: FileNotFoundException: " + key + ":" + pathname);
               }

          return(s);
          }
     }

//------------------------------------------------------------------------------
//  remove all cache items
//------------------------------------------------------------------------------
public void removeAll() {
     clear();
     }

//------------------------------------------------------------------------------
//  remove specific cache item
//------------------------------------------------------------------------------
public void remove(String key) {
     remove(key);
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public void setActive(boolean value) { active = value; }
public void setDebug(boolean value) { debug = value; }

public boolean getActive() { return(active); }
public boolean getDebug() { return(debug); }
}
