//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class InvisibleFormatSmart {

String                          s;
InvisibleVector                 sections;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleFormatSmart(String sArg) {
     s = sArg;
     sections = new InvisibleVector();
     }

//------------------------------------------------------------------------------
//  get coded text
//------------------------------------------------------------------------------
public String get() {
     //-------------------------------------------------------------------------
     //  make sure there is some content
     //-------------------------------------------------------------------------
     if(s == null) return(null);

     //-------------------------------------------------------------------------
     //  clean up line endings
     //-------------------------------------------------------------------------
     s = InvisibleUtil.serp(s,"\r\n","\n");
     s = InvisibleUtil.serp(s,"\r","\n");
     s = InvisibleUtil.serp(s,"\n\n","\r");

     //-------------------------------------------------------------------------
     //  process
     //-------------------------------------------------------------------------
     StringBuffer sb = new StringBuffer();
     boolean returnFound = true;
     boolean newlineFound = false;
     boolean saveCharacter = true;
     int unorderedListElementCount = 0;
     int orderedListElementCount = 0;
     boolean blockquote = false;
     boolean anchor = false;
     int anchorcount = 0;
     StringBuffer heading = null;
     char c;
     char previousCharacter = ' ';
     int size = s.length();

     for(int k=0;k<size;++k) {
          //--------------------------------------------------------------------
          //  get the next character
          //--------------------------------------------------------------------
          c = s.charAt(k);

          saveCharacter = true;

          //--------------------------------------------------------------------
          //  don't check for email if tagged
          //--------------------------------------------------------------------
          boolean checkForEmail = true;

          //--------------------------------------------------------------------
          //  unordered list element
          //--------------------------------------------------------------------
          if(c == '-') {
               if((returnFound) || (newlineFound)) {
                    if(++unorderedListElementCount == 1) {
                         sb.append("<ul>");
                         }
                    sb.append("<li>");
                    saveCharacter = false;
                    }

               checkForEmail = false;
               }

          //--------------------------------------------------------------------
          //  ordered list element
          //--------------------------------------------------------------------
          if(c == '*') {
               if((returnFound) || (newlineFound)) {
                    if(++orderedListElementCount == 1) {
                         sb.append("<ol>");
                         }
                    sb.append("<li>");
                    saveCharacter = false;
                    }

               checkForEmail = false;
               }

          //--------------------------------------------------------------------
          //  block quote
          //--------------------------------------------------------------------
          if(c == '>') {
               if(returnFound) {
                    blockquote = true;
                    sb.append("<blockquote>");
                    saveCharacter = false;
                    }

               checkForEmail = false;
               }

          //--------------------------------------------------------------------
          //  see if paragraph start
          //--------------------------------------------------------------------
          if(returnFound) {
               sb.append("<p>");
               }

          //--------------------------------------------------------------------
          //  anchor
          //--------------------------------------------------------------------
          if(c == '!') {
               if(returnFound) {
                    sb.append("<a name=\"anchor");
                    sb.append(++anchorcount);
                    sb.append("\">");
                    anchor = true;
                    heading = new StringBuffer();
                    saveCharacter = false;
                    }

               checkForEmail = false;
               }

          //--------------------------------------------------------------------
          //  see if continued paragraph
          //--------------------------------------------------------------------
          if(newlineFound) {
               sb.append(" ");
               }

          //--------------------------------------------------------------------
          //  look ahead for an email address indication
          //--------------------------------------------------------------------
          if((checkForEmail) && ((previousCharacter == ' ') || (returnFound) || (newlineFound))) {
               try {
                    boolean atFound = false;

                    int j;
                    for(j=k;j<(k+64);++j) {
                         char c2 = s.charAt(j);
                         if(c2 == ' ') break;
                         if(c2 == '\r') break;
                         if(c2 == '\n') break;
                         if(c2 == ',') break;
                         if(c2 == '@') atFound = true;
                         }

                    if((atFound) && (j < (k+64))) {
                         //-----------------------------------------------------
                         //  possible e-mail address
                         //-----------------------------------------------------
                         String email = s.substring(k,j);
                         if(email.indexOf(".") != -1) {
                              sb.append("<a href=\"mailto:");
                              sb.append(email);
                              sb.append("\">");
                              sb.append(email);
                              sb.append("</a>");
                              k += email.length();
                              --k;
                              saveCharacter = false;
                              }
                         }
                    }
               catch(StringIndexOutOfBoundsException e) {
                    }
               }

          //--------------------------------------------------------------------
          //  some sort of line ending
          //--------------------------------------------------------------------
          if(c == '\r') {
               if(heading != null) {
                    sections.set(heading.toString());
                    heading = null;
                    }

               if(unorderedListElementCount > 0) {
                    sb.append("</ul>");
                    }
               else {
                    if(orderedListElementCount > 0) {
                         sb.append("</ol>");
                         }
                    else {
                         if(blockquote) {
                              sb.append("<br></blockquote>");
                              }
                         else {
                              if(anchor) {
                                   sb.append("</a><br><br>");
                                   }
                              else {
                                   sb.append("<br><br>");
                                   }
                              }
                         }
                    }
               unorderedListElementCount = 0;
               orderedListElementCount = 0;
               anchor = false;
               returnFound = true;
               sb.append("\n");
               continue;
               }

          //--------------------------------------------------------------------
          //  single line ending
          //--------------------------------------------------------------------
          if(c == '\n') {
               if(anchor) {
                    sb.append("</a><br><br>");
                    }
               if(heading != null) {
                    sections.set(heading.toString());
                    heading = null;
                    }
               anchor = false;
               newlineFound = true;
               sb.append("\n");
               continue;
               }

          //--------------------------------------------------------------------
          //  done with line endings
          //--------------------------------------------------------------------
          newlineFound = false;
          returnFound = false;

          //--------------------------------------------------------------------
          //  see if http:// reference
          //--------------------------------------------------------------------
          try {
               if(s.substring(k,k+7).equals("http://")) {
                    int indexend = s.indexOf(" ",k);
                    int indexendcomma = s.indexOf(",",k);
                    if((indexendcomma != -1) && (indexendcomma < indexend)) indexend = indexendcomma;
                    int indexendreturn = s.indexOf("\r",k);
                    if((indexendreturn != -1) && (indexendreturn < indexend)) indexend = indexendreturn;
                    int indexendnewline = s.indexOf("\n",k);
                    if((indexendnewline != -1) && (indexendnewline < indexend)) indexend = indexendnewline;

                    if(indexend == -1) indexend = size;

                    String uri = s.substring(k,indexend);

                    sb.append("<a href=\"");
                    sb.append(uri);
                    sb.append("\" target=\"_blank\">");
                    while(k < indexend) sb.append(s.charAt(k++));
                    --k;
                    sb.append("</a>");
                    saveCharacter = false;
                    }
               }
          catch(StringIndexOutOfBoundsException e) {
               }

          //--------------------------------------------------------------------
          //  remember previous character
          //--------------------------------------------------------------------
          previousCharacter = c;

          //--------------------------------------------------------------------
          //  save character if appropriate
          //--------------------------------------------------------------------
          if(saveCharacter) {
               if(heading != null) heading.append(c);
               sb.append(c);
               }
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public InvisibleVector getSections() { return(sections); }
}
