//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.util.Hashtable;
import java.util.Enumeration;

public class InvisibleHashtable extends Hashtable {

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleHashtable() {
     super();
     }

//------------------------------------------------------------------------------
//  set hashtable element
//------------------------------------------------------------------------------
public Object setHashtableElement(Object key,Object element) {
     return(put(key,element));
     }

//------------------------------------------------------------------------------
//  set
//------------------------------------------------------------------------------
public Object set(Object key,Object element) {
     return(put(key,element));
     }

//------------------------------------------------------------------------------
//  set string--blank if null
//------------------------------------------------------------------------------
public Object setString(Object key,String s) {
     if(s == null) return(put(key,""));
     return(put(key,s));
     }

//------------------------------------------------------------------------------
//  get hashtable element
//------------------------------------------------------------------------------
public Object getHashtableElement(Object key) {
     return(get(key));
     }

//------------------------------------------------------------------------------
//  get string
//------------------------------------------------------------------------------
public String getString(String key) {
     if(key == null) return(null);
     return((String)(get(key)));
     }

//------------------------------------------------------------------------------
//  key vector
//------------------------------------------------------------------------------
public InvisibleVector getKeys() {
     InvisibleVector v = new InvisibleVector();

     Enumeration keys = keys();

     while(keys.hasMoreElements()) {
          v.addElement(keys.nextElement());
          }

     return(v);
     }

//------------------------------------------------------------------------------
//  object vector
//------------------------------------------------------------------------------
public InvisibleVector getVector() {
     InvisibleVector keys = getKeys();

     InvisibleVector v = new InvisibleVector();
     int size = keys.size();

     for(int k=0;k<size;++k) {
          v.set(get(keys.getString(k)));
          }

     return(v);
     }

//------------------------------------------------------------------------------
//  key vector
//------------------------------------------------------------------------------
public InvisibleVector getKeysStartsWith(String s) {
     InvisibleVector v = new InvisibleVector();

     Enumeration keys = keys();

     while(keys.hasMoreElements()) {
          String key = (String)(keys.nextElement());
          if(key.startsWith(s)) {
               v.addElement(key);
               }
          }

     return(v);
     }

//------------------------------------------------------------------------------
//  key vector
//------------------------------------------------------------------------------
public InvisibleVector getKeysEndsWith(String s) {
     InvisibleVector v = new InvisibleVector();

     Enumeration keys = keys();

     while(keys.hasMoreElements()) {
          String key = (String)(keys.nextElement());
          if(key.endsWith(s)) {
               v.addElement(key);
               }
          }

     return(v);
     }

//------------------------------------------------------------------------------
//  key vector
//------------------------------------------------------------------------------
public InvisibleVector getKeysIncludesString(String s) {
     InvisibleVector v = new InvisibleVector();

     Enumeration keys = keys();

     while(keys.hasMoreElements()) {
          String key = (String)(keys.nextElement());
          if(key.indexOf(s) != -1) {
               v.addElement(key);
               }
          }

     return(v);
     }

//------------------------------------------------------------------------------
//  clear
//------------------------------------------------------------------------------
public void clear() {
     Enumeration keys = keys();

     while(keys.hasMoreElements()) {
          remove(keys.nextElement());
          }
     }
}
