//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletOutputStream;
import java.io.FileInputStream;
import java.io.IOException;

public class InvisiblePage {

HttpServletResponse             response;
String                          contentType;
ServletOutputStream             out;

//------------------------------------------------------------------------------
//  null constructor
//------------------------------------------------------------------------------
public InvisiblePage() {
     contentType = null;
     out = null;
     }

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisiblePage(HttpServletResponse responseArg) throws IOException {
     this(responseArg,null);
     }

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisiblePage(HttpServletResponse responseArg,String contentTypeArg) throws IOException {
     response = responseArg;

     if(contentType != null) setContentType(contentTypeArg);
     }

//------------------------------------------------------------------------------
//  set default content type
//------------------------------------------------------------------------------
public void setContentType() {
     setContentType("text/html");
     }

//------------------------------------------------------------------------------
//  set content type
//------------------------------------------------------------------------------
public void setContentType(String contentTypeArg) {
     try {
          contentType = contentTypeArg;
          response.setContentType(contentType);
          out = response.getOutputStream();
          }
     catch(IOException e) {
          }
     }

//------------------------------------------------------------------------------
//  send
//------------------------------------------------------------------------------
public boolean send(String uri) {
     return(send(uri,"text/html"));
     }

//------------------------------------------------------------------------------
//  send
//------------------------------------------------------------------------------
public boolean send(String uri,String contentTypeArg) {
     if((uri == null) || (uri.length() == 0)) return(false);

     try {
          //--------------------------------------------------------------------
          //  set mime type
          //--------------------------------------------------------------------
          setContentType(contentTypeArg);

          //--------------------------------------------------------------------
          //  set mime content disposition
          //--------------------------------------------------------------------
System.err.println("DEBUG: URI GOING IN: " + uri);
          try {
               for(int k=uri.length()-1;k>0;--k) {
                    if(uri.charAt(k) == '/') {
                         String urifilename = InvisibleUtil.urldecode(uri.substring(++k));
System.err.println("DEBUG: DISPOSITION FILENAME: " + urifilename);
                         //DEBUG...old way...response.setHeader("Content-Disposition","attachment; filename="+urifilename+";");
                         response.setHeader("Content-Disposition","attachment; filename="+urifilename+";");
                         break;
                         }
                    }
               }
          catch(ArrayIndexOutOfBoundsException e) {
System.err.println("DEBUG: UNABLE TO SET THE CONTENT DISPOSITION");
               }

          //--------------------------------------------------------------------
          //  send local or redirect
          //--------------------------------------------------------------------
          if((uri.startsWith("file:")) || (uri.startsWith("FILE:"))) {
               //---------------------------------------------------------------
               //  local file--open the input stream
               //---------------------------------------------------------------
               FileInputStream f = new FileInputStream(uri.substring(5));

               int i;
               while((i = f.read()) != -1) {
                    out.print((char)(i & 0xff));
                    }

               f.close();
               }
          else {
               //---------------------------------------------------------------
               //  redirect
               //---------------------------------------------------------------
               response.sendRedirect(uri);
               }

          return(true);
          }
     catch(IOException e) {
          return(false);
          }
     }

//------------------------------------------------------------------------------
//  println
//------------------------------------------------------------------------------
public void println(String s) {
     try {
          if(contentType == null) setContentType();
          if(s != null) out.println(s);
          }
     catch(IOException e) {
          }
     }

//------------------------------------------------------------------------------
//  print
//------------------------------------------------------------------------------
public void print(String s) {
     try {
          if(contentType == null) setContentType();
          if(s != null) out.print(s);
          }
     catch(IOException e) {
          }
     }

//------------------------------------------------------------------------------
//  print with hashing
//------------------------------------------------------------------------------
public void print(String s,InvisibleHashtable h) {
     try {
          if(contentType == null) setContentType();
          if(s != null) out.print(InvisibleUtil.hashstring(s,h));
          }
     catch(IOException e) {
          }
     }

//------------------------------------------------------------------------------
//  println with hashing
//------------------------------------------------------------------------------
public void println(String s,InvisibleHashtable h) {
     try {
          if(contentType == null) setContentType();
          if(s != null) out.println(InvisibleUtil.hashstring(s,h));
          }
     catch(IOException e) {
          }
     }

//------------------------------------------------------------------------------
//  close
//------------------------------------------------------------------------------
public void close() {
     try {
     if(out != null) {
          out.flush();
          out.close();
          }
          }
     catch(IOException e) {
          }
     }
 
//------------------------------------------------------------------------------
//  finally
//------------------------------------------------------------------------------
//public void finalize() {
//     close();
//     }
}
