//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.IOException;

public class InvisibleServlet extends HttpServlet {

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleServlet() {
     super();
     }

//------------------------------------------------------------------------------
//  init
//------------------------------------------------------------------------------
public void init(ServletConfig config) throws ServletException {
     super.init(config);
     }

//------------------------------------------------------------------------------
//  get method
//------------------------------------------------------------------------------
protected void doGet(HttpServletRequest request,HttpServletResponse response) throws ServletException,IOException {
     doService(request,response);
     }

//------------------------------------------------------------------------------
//  post method
//------------------------------------------------------------------------------
protected void doPost(HttpServletRequest request,HttpServletResponse response) throws ServletException,IOException {
     doService(request,response);
     }

//------------------------------------------------------------------------------
//  service method to overload
//------------------------------------------------------------------------------
protected void doService(HttpServletRequest request,HttpServletResponse response) throws ServletException,IOException {
     InvisiblePage page = new InvisiblePage(response);

     page.println("InvisibleServlet Running");
     } 

//------------------------------------------------------------------------------
//  build complete url from request
//------------------------------------------------------------------------------
static public String url(HttpServletRequest request) {
     StringBuffer sb = new StringBuffer();
     sb.append(request.getScheme());
     sb.append("://");   
     sb.append(request.getServerName());
     int port = request.getServerPort();
     if(port != 80) {    
          sb.append(":");
          sb.append(String.valueOf(port));
          }
     sb.append(request.getRequestURI());
     String queryString = request.getQueryString();
     if((queryString != null) && (queryString.length() > 0)) {
          sb.append("?");
          sb.append(queryString);
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  get stack trace as a string
//------------------------------------------------------------------------------
static public String getStackTrace(Exception e) {
     return(InvisibleUtil.getStackTrace(e));
     }

//------------------------------------------------------------------------------
//  get a cookie by name creating if necessary
//------------------------------------------------------------------------------
public String getCookie(HttpServletRequest request,HttpServletResponse response,String name) {
     //-------------------------------------------------------------------------
     //  search cookie list
     //-------------------------------------------------------------------------
     Cookie[] cookies = request.getCookies();

     if(cookies != null) {
          for(int k=0;k<cookies.length;++k) {
               if(cookies[k].getName().equals(name)) {
                    //------------------------------------------------
                    //  cookie available
                    //------------------------------------------------
                    return(new String(cookies[k].getValue()));
                    }
               }
          }

     //-------------------------------------------------------------------------
     //  no cookie available--make a new one
     //-------------------------------------------------------------------------
     String cookieValue = String.valueOf(System.currentTimeMillis());
     Cookie cookie = new Cookie(name,cookieValue);

     cookie.setComment("Invisible Worlds Session Tracking");
     cookie.setMaxAge(-1);
     response.addCookie(cookie);

     return(cookieValue);
     }

//------------------------------------------------------------------------------
//  profiling tool
//------------------------------------------------------------------------------
public long getTicks(String message,long ticks) {
     long newticks;
     if(ticks <= 0) {
          ticks = System.currentTimeMillis();
          newticks = ticks;
          }
     else {
          newticks = System.currentTimeMillis();
          }
     System.err.println(message+" ("+String.valueOf(newticks-ticks)+" ms)");
     return(newticks);
     }
}
