//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.lang.StringIndexOutOfBoundsException;

public class InvisibleTokenizer {
 
static final int                        NONE = 0;
static final int                        ELEMENT = 1;
static final int                        ATTRIBUTE = 2;
static final int                        CLOSINGELEMENT = 3;

StringBuffer                            config;
int                                     type;
String                                  key;
String                                  value;
int                                     index;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleTokenizer(String s) {
     this(new StringBuffer(s));
     }

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleTokenizer(StringBuffer configArg) {
     config = configArg;
     type = NONE;
     key = null;
     value = null;
     index = 0;

     skipWhiteSpace();
     }

//------------------------------------------------------------------------------
//  next
//------------------------------------------------------------------------------
public boolean next() throws InvisibleException {
     //-------------------------------------------------------------------------
     //  see if done or starting another token
     //-------------------------------------------------------------------------
     if(index >= config.length()) {
          return(false);
          }

     char c = config.charAt(index);

     if(c == '<') {
          //--------------------------------------------------------------------
          //  next token is a new element
          //--------------------------------------------------------------------
          ++index;

          //--------------------------------------------------------------------
          //  for this purpose skip any <!...> and </...> elements
          //--------------------------------------------------------------------
          c = config.charAt(index);

          if(c == '!') {
               int length = config.length();
               while((++index < length) && (config.charAt(index) != '<'));
               if(index >= length) return(false);
               ++index;
               }

          return(getElement(false));
          }
     else {
          //--------------------------------------------------------------------
          //  next token is an attribute
          //--------------------------------------------------------------------
          return(getAttribute());
          }
     }

//------------------------------------------------------------------------------
//  get an element
//------------------------------------------------------------------------------
private boolean getElement(boolean equalSignOk) throws InvisibleException {
     try {
          if(config.charAt(index) == '/') {
               type = CLOSINGELEMENT;
               ++index;
               }
          else {
               type = ELEMENT;
               }

          StringBuffer sb = new StringBuffer();

          int length = config.length();
          char c = 0;

          while(index < length) {
               c = config.charAt(index++);
               if(c == ' ') break;
               if(c == '\n') break;
               if(c == '\r') break;
               if(c == '\t') break;
               if(c == '/') break;
               if(c == '>') break;
               if(c == '=') break;
               sb.append(c);
               }

          if((c != '>') && (index >= length)) {
               return(false);
               }

          if(c == '=') {
               if(!(equalSignOk)) {
                    throw new InvisibleException("InvisibleTokenizer: Element Ended Incorrectly");
                    }

               --index;
               }

          key = sb.toString();
          value = null;

          skipWhiteSpace();

          return(true);
          }
     catch(StringIndexOutOfBoundsException e) {
          throw new InvisibleException("InvisibleTokenizer: Element Ended Prematurely");
          }
     }

//------------------------------------------------------------------------------
//  get an attribute
//------------------------------------------------------------------------------
private boolean getAttribute() throws InvisibleException {
     try {
          getElement(true);

          char c = config.charAt(index);

          if(c != '=') {
               throw new InvisibleException("InvisibleTokenizer: Missing Attribute Value");
               }

          ++index;

          skipWhiteSpace();

          c = config.charAt(index);

          if((c == '"') || (c == '\'')) {
               char delimiter = c;

               StringBuffer sb = new StringBuffer();

               ++index;

               while((c = config.charAt(index++)) != delimiter) {
                    sb.append(c);
                    }

               value = sb.toString();
               }
          else {
               throw new InvisibleException("InvisibleTokenizer: Missing Attribute Quote");
               }

          type = ATTRIBUTE;

          skipWhiteSpace();

          return(true);
          }
     catch(InvisibleException e) {
          throw new InvisibleException("InvisibleTokenizer: Attribute Ended Prematurely");
          }
     catch(StringIndexOutOfBoundsException e) {
          throw new InvisibleException("InvisibleTokenizer: Attribute Ended Prematurely");
          }
     }

//------------------------------------------------------------------------------
//  skip all white space
//------------------------------------------------------------------------------
private void skipWhiteSpace() {
     int length = config.length();

     for(;index < length;++index) {
          char c = config.charAt(index);
          if(c == ' ') continue;
          if(c == '\n') continue;
          if(c == '\r') continue;
          if(c == '\t') continue;
          if(c == '/') continue;
          if(c == '>') continue;
          if(c == '?') continue;
          break;
          }
     }

//------------------------------------------------------------------------------
//  get type
//------------------------------------------------------------------------------
public boolean isNone() { return(type == NONE); }
public boolean isElement() { return(type == ELEMENT); }
public boolean isAttribute() { return(type == ATTRIBUTE); }
public boolean isClosingElement() { return(type == CLOSINGELEMENT); }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public String getKey() { return(key); }
public String getValue() { return(value); }
}
