//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.util.StringTokenizer;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

public class InvisibleUrlRequest {

String                                  username;
String                                  password;
String                                  exception;
long                                    responseMilliseconds;
boolean                                 debug;

//------------------------------------------------------------------------------
//  construct
//------------------------------------------------------------------------------
public InvisibleUrlRequest() {
     username = null;
     password = null;
     exception = null;
     responseMilliseconds = 0;
     debug = false;
     }

//------------------------------------------------------------------------------
//  get ssi
//------------------------------------------------------------------------------
public String getSsi(String ssi) throws InvisibleException {
     //-------------------------------------------------------------------------
     //  ssi uri
     //-------------------------------------------------------------------------
     StringBuffer url = new StringBuffer();

     int paramCount = 0;

     //-------------------------------------------------------------------------
     //  parse the overall
     //-------------------------------------------------------------------------
     StringTokenizer parse = new StringTokenizer(ssi,"<>\n\r");

     while(parse.hasMoreTokens()) {
          //--------------------------------------------------------------------
          //  get token
          //--------------------------------------------------------------------
          String token = parse.nextToken();

          //--------------------------------------------------------------------
          //  code
          //--------------------------------------------------------------------
          if(token.startsWith("servlet")) {
               int indexStart = token.indexOf("code=");

               if(indexStart == -1) {
                    throw new InvisibleException("InvisibleUrlRequest: Invalid SSI Code");
                    }

               indexStart += 5; //length of code=
               int indexEnd = token.length();

               url.append(InvisibleUtil.unquote(token.substring(indexStart,indexEnd)));
               continue;
               }

          //--------------------------------------------------------------------
          //  parameter
          //--------------------------------------------------------------------
          if(token.startsWith("param")) {
               //---------------------------------------------------------------
               //  get name/value pair
               //---------------------------------------------------------------
               int indexNameStart = token.indexOf("name=");
               if(indexNameStart == -1) {
                    throw new InvisibleException("InvisibleUrlRequest: Invalid Parameter Name");
                    }
               indexNameStart += 5; //length of name=

               int indexNameEnd = token.indexOf(" ",indexNameStart);
               if(indexNameEnd == -1) {
                    throw new InvisibleException("InvisibleUrlRequest: Invalid Parameter Name End");
                    }

               int indexValueStart = token.indexOf("value=",indexNameEnd);
               if(indexValueStart == -1) {
                    throw new InvisibleException("InvisibleUrlRequest: Invalid Parameter Value");
                    }
               indexValueStart += 6; //length of value=
               int indexValueEnd = token.length();

               String name = InvisibleUtil.unquote(token.substring(indexNameStart,indexNameEnd));
               String value = InvisibleUtil.unquote(token.substring(indexValueStart,indexValueEnd));

               //---------------------------------------------------------------
               //  add parameter to url
               //---------------------------------------------------------------
               if(paramCount++ == 0) {
                    url.append("?");
                    }
               else {
                    url.append("&");
                    }

               url.append(name);
               url.append("=");
               url.append(value);
               continue;
               }

          //--------------------------------------------------------------------
          //  servlet end
          //--------------------------------------------------------------------
          if(token.startsWith("/servlet")) {
               break;
               }

          //--------------------------------------------------------------------
          //  unknown token
          //--------------------------------------------------------------------
          throw new InvisibleException("InvisibleUrlRequest: Invalid SSI Token");
          }

     //-------------------------------------------------------------------------
     //  return assembled url
     //-------------------------------------------------------------------------
     System.err.println("InvisibleUrlRequest: " + url.toString());

     return(get(url.toString()));
     }

//------------------------------------------------------------------------------
//  get
//------------------------------------------------------------------------------
public String get(String url) {
     try {
          //--------------------------------------------------------------------
          //  clear out last exception
          //--------------------------------------------------------------------
          exception = null;

          //--------------------------------------------------------------------
          //  mark start of request
          //--------------------------------------------------------------------
          responseMilliseconds = 0;

          long startMilliseconds = System.currentTimeMillis();

          //--------------------------------------------------------------------
          //  instantiate a connection
          //--------------------------------------------------------------------
          if(debug) System.err.println("InvisibleUrlRequest: URL: " + url);

          URL urlInstance = new URL(url);

          URLConnection connection = urlInstance.openConnection();
          connection.setUseCaches(false);

          //--------------------------------------------------------------------
          //  instantiate a connection
          //--------------------------------------------------------------------
          if((username != null) && (password != null)) {
               String authorization = InvisibleConverter.encode(username+":"+password);
               if(debug) System.err.println("InvisibleUrlRequest: authorization: " + authorization);
               connection.setRequestProperty ("Authorization", "Basic " + authorization);
               }
          else {
               if(debug) System.err.println("InvisibleUrlRequest: No Username/Password");
               }

          //--------------------------------------------------------------------
          //  open the input stream
          //--------------------------------------------------------------------
          InputStream stream = connection.getInputStream();
          BufferedReader in = new BufferedReader(new InputStreamReader(stream));

          //--------------------------------------------------------------------
          //  get the results
          //--------------------------------------------------------------------
          if(debug) System.err.println("InvisibleUrlRequest: Get Stream");

          StringBuffer results = new StringBuffer();
          String s;

          while((s = in.readLine()) != null) {
               results.append(s);
               results.append("\n");
               }

          //--------------------------------------------------------------------
          //  calculate response
          //--------------------------------------------------------------------
          responseMilliseconds = System.currentTimeMillis() - startMilliseconds;

          if(debug) System.err.println("InvisibleUrlRequest: Response Time: " + responseMilliseconds);

          //--------------------------------------------------------------------
          //  return results
          //--------------------------------------------------------------------
          if(debug) System.err.println("InvisibleUrlRequest: Results Length: " + results.length());

          return(results.toString());
          }
     catch(MalformedURLException e) {
          exception = e.toString();
          if(debug) System.err.println("InvisibleUrlRequest: MalformedURLException: " + exception);
          return(null);
          }
     catch(IOException e) {
          exception = e.toString();
          if(debug) System.err.println("InvisibleUrlRequest: IOException: " + exception);
          return(null);
          }
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public boolean isDebug() { return(debug); }

public void setUsername(String value) { username = value; }
public void setPassword(String value) { password = value; }
public void setException(String value) { exception = value; }
public void setDebug(boolean value) { debug = value; }

public String getUsername() { return(username); }
public String getPassword() { return(password); }
public String getException() { return(exception); }
public long getResponseMilliseconds() { return(responseMilliseconds); }
public boolean getDebug() { return(debug); }
}
