//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.util.StringTokenizer;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.IOException;

public class InvisibleUtil {

//------------------------------------------------------------------------------
//  instantiate a class by name
//------------------------------------------------------------------------------
static Object getInstance(String classname) throws InvisibleException {
     if(classname == null) {
          throw new InvisibleException("InvisibleUtil: Null Class Name");
          }

     try {
          Class getClass = Class.forName(classname);
          return(getClass.newInstance());
          }
     catch(java.lang.ClassNotFoundException e) {
          throw new InvisibleException("InvisibleUtil: Class Not Found "+classname+": "+e.toString());
          }
     catch(java.lang.IllegalAccessException e) {
          throw new InvisibleException("InvisibleUtil: Illegal Access To "+classname+": "+e.toString());
          }
     catch(java.lang.InstantiationException e) {
          throw new InvisibleException("InvisibleUtil: Unable To Instantiate "+classname+": "+e.toString());
          }
     }

//------------------------------------------------------------------------------
//  write specified file with the specified content
//------------------------------------------------------------------------------
static public boolean writeContent(String pathname,String content) {
     try {
          //--------------------------------------------------------------------
          //  create any necessary directories
          //--------------------------------------------------------------------
          File file = new File(pathname);
          File parent = new File(file.getParent());

          parent.mkdirs();

          //--------------------------------------------------------------------
          //  write the content
          //--------------------------------------------------------------------
          FileOutputStream f = new FileOutputStream(pathname);
          PrintWriter out = new PrintWriter(f);

          out.print(content);

          //--------------------------------------------------------------------
          //  close
          //--------------------------------------------------------------------
          out.close();
          f.close();

          return(true);
          }
     catch(IOException e) {
          return(false);
          }
     }

//------------------------------------------------------------------------------
//  determine if string designates yes
//------------------------------------------------------------------------------
static public boolean yes(String valueArg) {
     String value = valueArg.toLowerCase();

     if(value.equals("checked")) return(true);
     if(value.equals("ok")) return(true);
     if(value.equals("on")) return(true);
     if(value.equals("required")) return(true);
     if(value.equals("t")) return(true);
     if(value.equals("true")) return(true);
     if(value.equals("y")) return(true);
     if(value.equals("yes")) return(true);

     return(false);
     }

//------------------------------------------------------------------------------
//  determine if string designates no
//------------------------------------------------------------------------------
static public boolean no(String valueArg) {
     return(!(yes(valueArg)));
     }

//------------------------------------------------------------------------------
//  determine if boolean is true or false
//------------------------------------------------------------------------------
static public String trueorfalse(boolean b) {
     if(b) return("true");
     return("false");
     }

//------------------------------------------------------------------------------
//  parse string or tokenize
//------------------------------------------------------------------------------
static public InvisibleVector parse(String s,String delim) {
     InvisibleVector v = new InvisibleVector();

     StringTokenizer p = new StringTokenizer(s,delim);

     while(p.hasMoreTokens()) {
          v.set(p.nextToken());
          }

     return(v);
     }

//------------------------------------------------------------------------------
//  parse property name allowing periods in the name
//------------------------------------------------------------------------------
static public InvisibleVector parseProperty(String key) {
     //-------------------------------------------------------------------------
     //  standard parse with period delimiter
     //-------------------------------------------------------------------------
     if(key == null) return(null);

     InvisibleVector parse = parse(key,".");

     int size = parse.size();

     if(size < 2) {
          return(null);
          }

     //-------------------------------------------------------------------------
     //  no period in the key
     //-------------------------------------------------------------------------
     if(size <= 3) {
          return(parse);
          }

     //-------------------------------------------------------------------------
     //  build parsed results allowing periods within the key
     //-------------------------------------------------------------------------
     InvisibleVector v = new InvisibleVector();

     v.set(parse.get(0));
     StringBuffer sb = new StringBuffer();
     --size;
     for(int k=1;k<size;++k) {
          if(k > 1) sb.append(".");
          sb.append((String)(parse.get(k)));
          }
     v.set(sb.toString());
     v.set(parse.get(size));
 
     return(v);
     }

//------------------------------------------------------------------------------
//  parse integer
//------------------------------------------------------------------------------
static public int parseInt(String value) {
     try {
          return(Integer.parseInt(value));
          }
     catch(NumberFormatException e) {
          return(0);
          }
     }

//------------------------------------------------------------------------------
//  parse long
//------------------------------------------------------------------------------
static public long parseLong(String value) {
     try {
          return(Long.parseLong(value));
          }
     catch(NumberFormatException e) {
          return(0L);
          }
     }

//------------------------------------------------------------------------------
//  parse float
//------------------------------------------------------------------------------
static public float parseFloat(String value) {
     try {
          return(Float.parseFloat(value));
          }
     catch(NumberFormatException e) {
          return(0.0f);
          }
     }

//------------------------------------------------------------------------------
//  parse double
//------------------------------------------------------------------------------
static public double parseDouble(String value) {
     try {
          return(Double.parseDouble(value));
          }
     catch(NumberFormatException e) {
          return(0.0);
          }
     }

//------------------------------------------------------------------------------
//  size text
//------------------------------------------------------------------------------
static public String sizeText(int size) {
     return(sizeText((long)(size)));
     }

//------------------------------------------------------------------------------
//  size text
//------------------------------------------------------------------------------
static public String sizeText(long size) {
     StringBuffer sb = new StringBuffer(); 

     if(size >= (1024 * 1024)) {
          float f = ((float)(size)) / (1024 * 1024);
          long k = (long)(f * 100.0);
          sb.append(String.valueOf(((float)(k))/100.0));
          sb.append(" megabytes");
          }
     else {
          if(size >= 1024) {
               float f = ((float)(size)) / (1024);
               long k = (long)(f * 100.0);
               sb.append(String.valueOf(((float)(k))/100.0));
               sb.append(" Kbytes");
               }
          else {
               sb.append(String.valueOf(size));
               sb.append(" bytes");
               }
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  unquote string
//------------------------------------------------------------------------------
static public String unquote(String s) throws InvisibleException {
     if(s == null) return(null);

     s = s.trim();

     int length = s.length();

     if(length == 0) return(s);

     if(s.charAt(0) == '\"') {
          if((length == 1) || (s.charAt(--length) != '\"')) {
               throw new InvisibleException("InvisibleUtil: Improperly Quoted String");
               }
          return(s.substring(1,length));
          }

     return(s);
     }

//------------------------------------------------------------------------------
//  hash string
//------------------------------------------------------------------------------
static public String hashstring(String s,InvisibleHashtable h) {
     return(hashstring(s,h,false));
     }

//------------------------------------------------------------------------------
//  hash selection options given key and option content
//------------------------------------------------------------------------------
static void hashoptions(InvisibleHashtable hash,String key,StringBuffer options,String selectedKey) {
     //-------------------------------------------------------------------------
     //  normal option list
     //-------------------------------------------------------------------------
     hash.set(key,options.toString());
 
     //-------------------------------------------------------------------------
     //  "all" option list
     //-------------------------------------------------------------------------
     StringBuffer optionAll = new StringBuffer(options.toString());
 
     optionAll.insert(0,"<option value=\"All\">All");
     hash.set(key+"-all",optionAll.toString());
 
     //-------------------------------------------------------------------------
     //  "none" option list
     //-------------------------------------------------------------------------
     StringBuffer optionNone = new StringBuffer(options.toString());
 
     optionNone.insert(0,"<option value=\"None\">None");
     hash.set(key+"-none",optionNone.toString());
 
     //-------------------------------------------------------------------------
     //  "both" option list
     //-------------------------------------------------------------------------
     StringBuffer optionBoth = new StringBuffer(options.toString());
 
     optionBoth.insert(0,"<option value=\"Both\">Both");
     hash.set(key+"-both",optionBoth.toString());
     }

//------------------------------------------------------------------------------
//  hash selection item
//------------------------------------------------------------------------------
static public String hashitem(String key,String selectedKey) {
     StringBuffer sb = new StringBuffer();
     sb.append("<option value=\"");
     sb.append(key);
     sb.append("\"");
     if((selectedKey != null) && (key.equalsIgnoreCase(selectedKey))) sb.append(" selected");
     sb.append(">");
     sb.append(key);
     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  hash string
//------------------------------------------------------------------------------
public final static int                 encodingNone = 0;
public final static int                 encodingJavascript = 1;
public final static int                 encodingUrl = 2;
public final static int                 encodingUrlDecode = 3;
public final static int                 encodingLineEndingBr = 4;
public final static int                 encodingLineEndingBrWithLineLength = 5;
public final static int                 encodingLineEndingWrap = 6;
public final static int                 encodingSmart = 7;
public final static int                 encodingDate = 8;
public final static int                 encodingMulti = 9;

static public String hashstring(String s,InvisibleHashtable h,boolean showNonMatchingKeys) {
     //-------------------------------------------------------------------------
     //  ignore if no string or hashtable
     //-------------------------------------------------------------------------
     if(s == null) return(s);
     if(h == null) return(s);

     //-------------------------------------------------------------------------
     //  scan for matches
     //-------------------------------------------------------------------------
     StringBuffer sb = new StringBuffer();
     int length = s.length();
     int brlinelength = 64;
     String dateModifier = null;
     char c;

     for(int k=0;k<length;++k) {
          //--------------------------------------------------------------------
          //  bypass characters preceded by backslash
          //--------------------------------------------------------------------
          if(s.charAt(k) == '\\') {
               //---------------------------------------------------------------
               //  if a leftover backslash on the end leave it
               //---------------------------------------------------------------
               if(++k >= s.length()) {
                    sb.append("\\");
                    break;
                    }

               sb.append(s.charAt(k));
               continue;
               }

          //--------------------------------------------------------------------
          //  normal character if not start of percent sign delimiter
          //--------------------------------------------------------------------
          if((c = s.charAt(k)) != '%') {
               sb.append(c);
               continue;
               }

          //--------------------------------------------------------------------
          //  extract potential key
          //--------------------------------------------------------------------
          StringBuffer key = new StringBuffer();
          boolean encodedCharacterFound = false;

          for(int i=k+1;i<length;++i) {
               if((c = s.charAt(i)) == '%') break;
               if(c == '\r') break;
               if(c == '\n') break;
               if(c == '\t') break;
               if(c == ' ') break;
               if(c == '<') break;
               if(c == '>') break;
               if(c == '&') break;
               if(c == '=') break;
               if(c == '\"') break;
               key.append(c);

               //---------------------------------------------------------------
               //  allow encoded spaces and % in the string
               //---------------------------------------------------------------
               if(key.length() == 2) {
                    String encodedchar = key.toString();
                    if((encodedchar.equals("20")) || (encodedchar.equals("25"))) {
                         k += 2;
                         encodedCharacterFound = true;
                         break;
                         }
                    }
               }

          //--------------------------------------------------------------------
          //  just save encoded characters
          //--------------------------------------------------------------------
          if(encodedCharacterFound) {
               sb.append("%");
               sb.append(key.toString());
               continue;
               }

          //--------------------------------------------------------------------
          //  see if complete tag
          //--------------------------------------------------------------------
          if(c == '%') {
               //---------------------------------------------------------------
               //  determine if any encoding needed on output
               //---------------------------------------------------------------
               int encoding = encodingNone;

               //---------------------------------------------------------------
               //  see if encoded key
               //---------------------------------------------------------------
               String refinedKey = key.toString();

               if(refinedKey.endsWith("-js")) {
                    //----------------------------------------------------------
                    //  javascript encoded key
                    //----------------------------------------------------------
                    encoding = encodingJavascript;
                    refinedKey = refinedKey.substring(0,refinedKey.length()-3);
                    }
               else {
                    if(refinedKey.endsWith("-url")) {
                         //-----------------------------------------------------
                         //  url encoded key
                         //-----------------------------------------------------
                         encoding = encodingUrl;
                         refinedKey = refinedKey.substring(0,refinedKey.length()-4);
                         }
                    else {
                         if(refinedKey.endsWith("-urldecode")) {
                              //------------------------------------------------
                              //  url decoded key
                              //------------------------------------------------
                              encoding = encodingUrlDecode;
                              refinedKey = refinedKey.substring(0,refinedKey.length()-10);
                              }
                         else {
                              if(refinedKey.endsWith("-multi")) {
                                   //-------------------------------------------
                                   //  multi decoding
                                   //-------------------------------------------
                                   encoding = encodingMulti;
                                   refinedKey = refinedKey.substring(0,refinedKey.length()-6);
                                   }
                              else {
                                   int dateindex;

                                   if((dateindex = refinedKey.indexOf("-date-")) != -1) {
                                        //--------------------------------------
                                        //  date modifier
                                        //--------------------------------------
                                        encoding = encodingDate;
                                        dateModifier = refinedKey.substring(dateindex+6);
                                        refinedKey = refinedKey.substring(0,dateindex);
                                        }
                                   else {
                                        if(refinedKey.endsWith("-wrap")) {
                                             //---------------------------------
                                             //  wrap line ending
                                             //---------------------------------
                                             encoding = encodingLineEndingWrap;
                                             refinedKey = refinedKey.substring(0,refinedKey.length()-5);
                                             }
                                        else {
                                             if(refinedKey.endsWith("-smart")) {
                                                  //----------------------------
                                                  //  smart encoding
                                                  //----------------------------
                                                  encoding = encodingSmart;
                                                  refinedKey = refinedKey.substring(0,refinedKey.length()-6);
                                                  }
                                             else {
                                                  if(refinedKey.endsWith("-br")) {
                                                       //-----------------------
                                                       //  br line ending
                                                       //-----------------------
                                                       encoding = encodingLineEndingBr;
                                                       refinedKey = refinedKey.substring(0,refinedKey.length()-3);
                                                       }
                                                  else {
                                                       int brindex;

                                                       if((brindex = refinedKey.indexOf("-br-")) != -1) {
                                                            //------------------
                                                            //  br line-length
                                                            //------------------
                                                            encoding = encodingLineEndingBrWithLineLength;
                                                            String count = refinedKey.substring(brindex+4);
                                                            refinedKey = refinedKey.substring(0,brindex);
                                                            brlinelength = parseInt(count);
                                                            }
                                                       }
                                                  }
                                             }
                                        }
                                   }
                              }
                         }
                    }

               //---------------------------------------------------------------
               //  lookup key
               //---------------------------------------------------------------
               String replacement;

               if((replacement = ((String)(h.get(refinedKey)))) == null) {
                    //----------------------------------------------------------
                    //  unknown key
                    //----------------------------------------------------------
                    if(showNonMatchingKeys) {
                         sb.append("%");
                         sb.append(key);
                         sb.append("%");
                         }
                    }
               else {
                    //----------------------------------------------------------
                    //  matching key
                    //----------------------------------------------------------
                    switch(encoding) {
                         case encodingNone :
                              sb.append(replacement);
                              break;
                         case encodingJavascript :
                              sb.append(jsencode(replacement));
                              break;
                         case encodingUrl :
                              sb.append(urlencode(replacement));
                              break;
                         case encodingUrlDecode :
                              sb.append(urldecode(replacement));
                              break;
                         case encodingLineEndingBr :
                              sb.append(brencode(replacement,32000));
                              break;
                         case encodingLineEndingBrWithLineLength :
                              sb.append(brencode(replacement,brlinelength));
                              break;
                         case encodingLineEndingWrap :
                              sb.append(wrapencode(replacement));
                              break;
                         case encodingSmart :
                              InvisibleFormatSmart smart = new InvisibleFormatSmart(replacement);
                              sb.append(smart.get());
                              break;
                         case encodingDate :
                              try {
                                   if((replacement != null) && (replacement.length() > 0)) {
                                        InvisibleDate date = new InvisibleDate(replacement);
                                        sb.append(date.get(dateModifier));
                                        }
                                   }
                              catch(InvisibleDateException e) {
                                   sb.append(replacement);
                                   }
                              break;
                         case encodingMulti :
                              if(replacement.endsWith(",")) {
                                   replacement = replacement.substring(0,replacement.length()-1);
                                   }
                              sb.append(replacement);

                              break;
                         }
                    }
               }
          else {
               //---------------------------------------------------------------
               //  unterminated key--do not treat as key
               //---------------------------------------------------------------
               sb.append("%");
               sb.append(key);
               if((k + key.length() + 1) < length) sb.append(c);
               }

          //--------------------------------------------------------------------
          //  skip key
          //--------------------------------------------------------------------
          k = k + key.length() + 1;
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  convert string to url format
//------------------------------------------------------------------------------
static public String urlencode(String s) {
     return(urlencode(s,false,false,false));
     }   

//------------------------------------------------------------------------------
//  urlencode
//------------------------------------------------------------------------------
static public String urlencode(String s,boolean nullEncode,boolean hyphenEncode,boolean ampersandEncode) {
     //-------------------------------------------------------------------------
     //  null case
     //-------------------------------------------------------------------------
     if((nullEncode) && (s.length() == 0)) {
          return(new String("%00"));
          }

     StringBuffer sb = new StringBuffer();

     for(int k=0;k<s.length();++k) {
          char c = s.charAt(k);
          switch(c) {
               case ' ' :
                    sb.append("%20");
                    break;
               case '\"' :
                    sb.append("%22");
                    break;
               case '#' :
                    sb.append("%23");
                    break;
               case '%' :
                    sb.append("%25");
                    break;
               case '-' :
                    if(hyphenEncode) {
                         sb.append("%2d");
                         }
                    else {
                         sb.append(c);
                         }
                    break;
               case '&' :
                    if(ampersandEncode) {
                         sb.append("%26");
                         }
                    else {
                         sb.append(c);
                         }
                    break;
               case '<' :
                    sb.append("%3c");
                    break;
               case '>' :
                    sb.append("%3e");
                    break;
               case '?' :
                    sb.append("%3f");
                    break;
               default :
                    sb.append(c);
                    break;
               }
          }
 
     return(sb.toString());
     }
 
//------------------------------------------------------------------------------
//  urldecode default behavior
//------------------------------------------------------------------------------
static public String urldecode(String s) {
     return(urldecode(s,false,false));
     }
 
//------------------------------------------------------------------------------
//  urldecode
//------------------------------------------------------------------------------
static public String urldecode(String s,boolean nullDecode,boolean colonDecode) {
     StringBuffer sb = new StringBuffer();
 
     for(int k=0;k<s.length();++k) {
          char c = s.charAt(k);
 
          if((colonDecode) && (c == ':') && ((k + 3) <= s.length())) {
               String sub = s.substring(k,k+3);
 
               if(sub.equals(":2f")) {
                    sb.append("/");
                    k += 2;
                    continue;
                    }
               }
 
          if((c == '%') && ((k + 3) <= s.length())) {
               String sub = s.substring(k,k+3).toLowerCase();
 
               if((nullDecode) && (sub.equals("%00"))) {
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%20")) { sb.append(" "); k += 2; continue; }
               if(sub.equals("%21")) { sb.append("!"); k += 2; continue; }
               if(sub.equals("%22")) { sb.append("\""); k += 2; continue; }
               if(sub.equals("%24")) { sb.append("$"); k += 2; continue; }
               if(sub.equals("%23")) {
                    sb.append("#");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%25")) {
                    sb.append("%");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%26")) {
                    sb.append("&");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%27")) {
                    sb.append("'");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%28")) {
                    sb.append("(");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%29")) {
                    sb.append(")");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%2a")) {
                    sb.append("*");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%2b")) {
                    sb.append("+");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%2c")) {
                    sb.append(",");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%2d")) {
                    sb.append("-");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%2e")) {
                    sb.append(".");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%2f")) {
                    sb.append("."); //really a slash...
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%3a")) {
                    sb.append("."); //really colon...
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%3b")) {
                    sb.append(";");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%3c")) {
                    sb.append("<");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%3d")) {
                    sb.append("=");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%3e")) {
                    sb.append(">");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%3f")) {
                    sb.append("?"); //used to be period for some reason
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%40")) {
                    sb.append("@");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%5b")) {
                    sb.append("[");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%5c")) {
                    sb.append("."); //really backslash...
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%5d")) {
                    sb.append("]");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%5e")) {
                    sb.append("^");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%5f")) {
                    sb.append("_");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%60")) {
                    sb.append("`");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%7b")) {
                    sb.append("{");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%7c")) {
                    sb.append("|");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%7d")) {
                    sb.append("}");
                    k += 2;
                    continue;
                    }
 
               if(sub.equals("%7e")) {
                    sb.append("~");
                    k += 2;
                    continue;
                    }
               }
 
          sb.append(c);
          } 

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  jsencode
//------------------------------------------------------------------------------
public static String jsencode(String s) {
     StringBuffer sb = new StringBuffer();

     for(int k=0;k<s.length();++k) {
          char c = s.charAt(k);

          switch(c) {
               case '\r' :
                    sb.append("\\r");
                    continue;
               case '\n' :
                    sb.append("\\n");
                    continue;
               case '\'' :
                    sb.append("\\'");
                    continue;
               } 

          sb.append(c);
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  convert string to <br> line ending
//------------------------------------------------------------------------------
static public String brencode(String s,int brlinelength) {
     //-------------------------------------------------------------------------
     //  tokenize
     //-------------------------------------------------------------------------
     StringTokenizer parse = new StringTokenizer(s," ,.-\n\r",true);

     //-------------------------------------------------------------------------
     //  build each line
     //-------------------------------------------------------------------------
     StringBuffer sb = new StringBuffer();
     int length = 0;

     while(parse.hasMoreTokens()) {
          String token = parse.nextToken();

          if((token.equals(" ")) || (token.equals(",")) || (token.equals(".")) || (token.equals("-"))) {
               ++length;
               sb.append(token);
               }
          else {
               if((token.equals("\n")) || (token.equals("\r"))) {
//INCOMPLETE...only do for new line?
                    if(token.equals("\n")) sb.append("<br>");
                    length = 0;
                    }
               else {
                    if((length + token.length()) > brlinelength) {
                         if(sb.length() > 0) sb.append("<br>");
                         length = 0;
                         }
                    }

//INCOMPLETE...does not handle long lines without a delimiter
               length += token.length();
               sb.append(token);
               }
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  wrap using <br> line ending
//------------------------------------------------------------------------------
static public String wrapencode(String s) {
     //-------------------------------------------------------------------------
     //  scan for line endings
     //-------------------------------------------------------------------------
     StringBuffer sb = new StringBuffer();

     int length = s.length();

     boolean previousReturn = false;
     boolean previousNewline = false;

     for(int k=0;k<length;++k) {
          char c = s.charAt(k);

          switch(c) {
               case '\r' :
                    if((previousReturn) || (previousNewline)) sb.append("<br><br>");
                    previousReturn = true;
                    sb.append("\r");
                    continue;
               case '\n' :
                    if(previousNewline) sb.append("<br><br>");
                    previousNewline = true;
                    sb.append("\n");
                    continue;
               } 

          previousReturn = false;
          previousNewline = false;
          sb.append(c);
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  textencode
//------------------------------------------------------------------------------
public static String textencode(String s) {
     StringBuffer sb = new StringBuffer();

     int length = s.length();

     for(int k=0;k<length;++k) {
          char c = s.charAt(k);

          switch(c) {
               case '\r' :
                    sb.append("\\r");
                    continue;
               case '\n' :
                    sb.append("\\n");
                    continue;
               case '\t' :
                    sb.append("\\t");
                    continue;
               case '\\' :
                    sb.append("\\\\");
                    continue;
               } 

          sb.append(c);
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  textdecode
//------------------------------------------------------------------------------
public static String textdecode(String s) {
     StringBuffer sb = new StringBuffer();

     int length = s.length();

     for(int k=0;k<length;++k) {
          char c = s.charAt(k);

          if(c == '\\') {
               if((++k) >= length) break;

               switch((c = s.charAt(k))) {
                    case 'r' :
                         sb.append("\r");
                         continue;
                    case 'n' :
                         sb.append("\n");
                         continue;
                    case 't' :
                         sb.append("\t");
                         continue;
                    case '\\' :
                         sb.append("\\");
                         continue;
                    default :
                         sb.append(c);
                    }

               continue;
               }

          sb.append(c);
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  serp
//------------------------------------------------------------------------------
public static String serp(String s,String search,String replace) {
     StringBuffer sb = new StringBuffer();

     int length = s.length();
     int lengthSearch = search.length();

     for(int k=0;k<length;++k) {
          //--------------------------------------------------------------------
          //  all done if search length exceeds remaining
          //--------------------------------------------------------------------
          if((k + lengthSearch) > length) {
               sb.append(s.substring(k,length));
               break;
               }

          //--------------------------------------------------------------------
          //  replace if search found
          //--------------------------------------------------------------------
          if(search.equals(s.substring(k,k+lengthSearch))) {
               sb.append(replace);
               k += (lengthSearch-1);
               }
          else {
               sb.append(s.charAt(k));
               }
          }

     return(sb.toString());
     }

//------------------------------------------------------------------------------ 
//  get attribute value 
//------------------------------------------------------------------------------ 
static public String getAttributeValue(String name,String s) { 
     try { 
          String nameEquals = name+"=";
          int index = s.indexOf(nameEquals); 
          if(index != -1) { 
               index += nameEquals.length(); 
               char delimiter = s.charAt(index++); 
               if((delimiter == '"') || (delimiter == '\'')) { 
                    int indexend = s.indexOf(delimiter,index);
                    if(index != -1) { 
                         return(s.substring(index,indexend)); 
                         } 
                    } 
               } 
 
          return(null); 
          } 
     catch(Exception e) { 
          return(null); 
          } 
     } 
 
//------------------------------------------------------------------------------
//  pad
//------------------------------------------------------------------------------
static public String pad(long l,int length,char c) {
     return(pad(String.valueOf(l),length,c));
     }
 
//------------------------------------------------------------------------------
//  pad
//------------------------------------------------------------------------------
static public String pad(int i,int length,char c) {
     return(pad(String.valueOf(i),length,c));
     }
 
//------------------------------------------------------------------------------
//  pad
//------------------------------------------------------------------------------
static public String pad(boolean b,int length,char c) {
     return(pad(((b)?"Yes":"No"),length,c));
     }
 
//------------------------------------------------------------------------------
//  pad
//------------------------------------------------------------------------------
static public String pad(String s,int length,char c) {
     if(s == null) return(null);
     StringBuffer sb = new StringBuffer();
     int padlength = length - s.length();
     while(padlength-- > 0) {
          sb.append(c);
          }
     sb.append(s);
     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  fill
//------------------------------------------------------------------------------
static public String fill(long l,int length,char c) {
     return(fill(String.valueOf(l),length,c));
     }
 
//------------------------------------------------------------------------------
//  fill
//------------------------------------------------------------------------------
static public String fill(int i,int length,char c) {
     return(fill(String.valueOf(i),length,c));
     }
 
//------------------------------------------------------------------------------
//  fill
//------------------------------------------------------------------------------
static public String fill(boolean b,int length,char c) {
     return(fill(((b)?"Yes":"No"),length,c));
     }
 
//------------------------------------------------------------------------------
//  fill
//------------------------------------------------------------------------------
static public String fill(String s,int length,char c) {
     if(s == null) return(null);
     StringBuffer sb = new StringBuffer(s);
     int filllength = length - s.length();
     while(filllength-- > 0) {
          sb.append(c);
          }
     return(sb.toString());
     }

//------------------------------------------------------------------------------
//  get stack trace as a string
//------------------------------------------------------------------------------
static public String getStackTrace(Exception e) {
     ByteArrayOutputStream bytes = new ByteArrayOutputStream();
     PrintWriter writer = new PrintWriter(bytes,true);
     e.printStackTrace(writer);
     return(bytes.toString());
     }

//------------------------------------------------------------------------------
//  static method: sleep specified milliseconds
//------------------------------------------------------------------------------
public static void sleep(long millis) {
     try {
          Thread.sleep(millis);
          } 
     catch(InterruptedException e) {
          } 
     }
 
//------------------------------------------------------------------------------
//  static method: sleep specified seconds
//------------------------------------------------------------------------------
public static void sleepSeconds(long seconds) {
     try {
          Thread.sleep(seconds*1000);
          }
     catch(InterruptedException e) {
          }
     }
}
