//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

public class InvisibleVector extends Vector {

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleVector() {
     super();
     }

//------------------------------------------------------------------------------
//  set
//------------------------------------------------------------------------------
public void set(Object o) {
     addElement(o);
     }

//------------------------------------------------------------------------------
//  set string
//------------------------------------------------------------------------------
public void setString(String s) {
     if(s == null) return;

     addElement(s);
     }

//------------------------------------------------------------------------------
//  set string unique
//------------------------------------------------------------------------------
public void setStringUnique(String s) {
     if(s == null) return;

     int index = find(s);

     if(index == -1) {
          addElement(s);
          }
     else {
          setElementAt(s,index);
          }
     }

//------------------------------------------------------------------------------
//  set strings unique after parsing string
//------------------------------------------------------------------------------
public void setStringUnique(String s,String delims) {
     try {
          StringTokenizer parse = new StringTokenizer(s,delims);
          while(parse.hasMoreTokens()) {
               setStringUnique(parse.nextToken());
               }
          }
     catch(NoSuchElementException e) {
          }
     }

//------------------------------------------------------------------------------
//  set strings after parsing string
//------------------------------------------------------------------------------
public void setString(String s,String delims) {
     try {
          StringTokenizer parse = new StringTokenizer(s,delims);
          while(parse.hasMoreTokens()) {
               setString(parse.nextToken());
               }
          }
     catch(NoSuchElementException e) {
          }
     }

//------------------------------------------------------------------------------
//  set numbered element
//------------------------------------------------------------------------------
public void set(Object o,int item) {
     setElementAt(o,item);
     }

//------------------------------------------------------------------------------
//  get
//------------------------------------------------------------------------------
public Object get(int index) {
     return(elementAt(index));
     }

//------------------------------------------------------------------------------
//  get as a string
//------------------------------------------------------------------------------
public String getString(int index) {
     return((String)(elementAt(index)));
     }

//------------------------------------------------------------------------------
//  get as an integer
//------------------------------------------------------------------------------
public int getInt(int index) {
     try {
          return(Integer.parseInt(getString(index)));
          }
     catch(NumberFormatException e) {
          return(0);
          }
     }

//------------------------------------------------------------------------------
//  get as a long
//------------------------------------------------------------------------------
public long getLong(int index) {
     try {
          return(Long.parseLong(getString(index)));
          }
     catch(NumberFormatException e) {
          return(0L);
          }
     }

//------------------------------------------------------------------------------
//  get message for html pate
//------------------------------------------------------------------------------
public String getMessage() {
     return(getMessage("<br>"));
     }

//------------------------------------------------------------------------------
//  get message for html pate
//------------------------------------------------------------------------------
public String getMessageText() {
     return(getMessage("\n"));
     }

//------------------------------------------------------------------------------
//  get message
//------------------------------------------------------------------------------
public String getMessage(String lineend) {
     int size = size();

     if(size == 0) return(new String(""));

     StringBuffer sb = new StringBuffer();

     for(int k=0;k<size;++k) {
          sb.append(getString(k));
          sb.append("<br>");
          }

     return(sb.toString());
     }   


//------------------------------------------------------------------------------
//  put
//------------------------------------------------------------------------------
public void put(Object o) {
     addElement(o);
     }

//------------------------------------------------------------------------------
//  find
//------------------------------------------------------------------------------
public int find(String key) {
     if(key == null) return(-1);

     int size = size();

     for(int k=0;k<size;++k) {
          if(((String)(elementAt(k))).equals(key)) return(k);
          }

     return(-1);
     }

//------------------------------------------------------------------------------
//  find
//------------------------------------------------------------------------------
public int find(InvisibleVector keyVector) {
     if(keyVector == null) return(-1);
     if(keyVector.size() == 0) return(-1);

     int size = keyVector.size();

     for(int k=0;k<size;++k) {
          String key = keyVector.getString(k);

          int index = find(key);

          if(index != -1) return(index);
          }

     return(-1);
     }

//------------------------------------------------------------------------------
//  flip
//------------------------------------------------------------------------------
public void flip(int item1,int item2) { 
     Object o = elementAt(item1);
     setElementAt(elementAt(item2),item1);
     setElementAt(o,item2);
     }
}
