//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class InvisibleVectorTable {

protected InvisibleVector               tableKey;
protected InvisibleVector               tableObject;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public InvisibleVectorTable() {
     tableKey = new InvisibleVector();
     tableObject = new InvisibleVector();
     }

//------------------------------------------------------------------------------
//  set
//------------------------------------------------------------------------------
synchronized public void set(String key,Object object) {
     //-------------------------------------------------------------------------
     //  set vectors
     //-------------------------------------------------------------------------
     tableKey.addElement(key);
     tableObject.addElement(object);
     }

//------------------------------------------------------------------------------
//  insert
//------------------------------------------------------------------------------
synchronized public void insert(String key,Object object) {
     //-------------------------------------------------------------------------
     //  set vectors at top of vectors
     //-------------------------------------------------------------------------
     insert(key,object,0);
     }

//------------------------------------------------------------------------------
//  insert
//------------------------------------------------------------------------------
synchronized public void insert(String key,Object object,int index) {
     //-------------------------------------------------------------------------
     //  set vectors
     //-------------------------------------------------------------------------
     tableKey.insertElementAt(key,index);
     tableObject.insertElementAt(object,index);
     }

//------------------------------------------------------------------------------
//  set unique
//------------------------------------------------------------------------------
synchronized public void setUnique(String key,Object object) {
     //-------------------------------------------------------------------------
     //  find key
     //-------------------------------------------------------------------------
     int item;

     if((item = tableKey.find(key)) == -1) {
          //--------------------------------------------------------------------
          //  new key
          //--------------------------------------------------------------------
          set(key,object);
          return;
          }

     //-------------------------------------------------------------------------
     //  replace existing key
     //-------------------------------------------------------------------------
     tableObject.setElementAt(object,item);
     }

//------------------------------------------------------------------------------
//  get
//------------------------------------------------------------------------------
synchronized public Object get(String key) {
     //-------------------------------------------------------------------------
     //  find key
     //-------------------------------------------------------------------------
     int item;

     if((item = tableKey.find(key)) == -1) {
          //--------------------------------------------------------------------
          //  not found
          //--------------------------------------------------------------------
          return(null);
          }

     //-------------------------------------------------------------------------
     //  found
     //-------------------------------------------------------------------------
     return(tableObject.elementAt(item));
     }

//------------------------------------------------------------------------------
//  get string
//------------------------------------------------------------------------------
public String getString(String key) {
      return((String)(get(key)));
      }

public String getString(int index) {
     return((String)(tableObject.elementAt(index)));
     }

//------------------------------------------------------------------------------
//  get key by index
//------------------------------------------------------------------------------
public String getKey(int index) {
     return((String)(tableKey.elementAt(index)));
     }

//------------------------------------------------------------------------------
//  get object by index
//------------------------------------------------------------------------------
public Object getObject(int index) {
     return(tableObject.elementAt(index));
     }

public Object get(int index) {
     return(getObject(index));
     }

//------------------------------------------------------------------------------
//  get vector of all matches
//------------------------------------------------------------------------------
synchronized public InvisibleVector getVector(String key) {
     //-------------------------------------------------------------------------
     //  scan and save matches
     //-------------------------------------------------------------------------
     InvisibleVector v = null;

     int size = tableKey.size();

     for(int k=0;k<size;++k) {
          if(((String)(tableKey.elementAt(k))).equals(key)) {
               if(v == null) v = new InvisibleVector();

               v.addElement(tableObject.elementAt(k));
               }
          }

     return(v);
     }

//------------------------------------------------------------------------------
//  remove
//------------------------------------------------------------------------------
synchronized public void remove(String key) {
     //-------------------------------------------------------------------------
     //  remove matches
     //-------------------------------------------------------------------------
     int size = tableKey.size();

     for(int k=(size-1);k>=0;--k) {
          if(((String)(tableKey.elementAt(k))).equals(key)) {
               tableKey.removeElementAt(k);
               tableObject.removeElementAt(k);
               }
          }
     }

//------------------------------------------------------------------------------
//  remove all
//------------------------------------------------------------------------------
synchronized public void removeAll() {
     //-------------------------------------------------------------------------
     //  remove all elements
     //-------------------------------------------------------------------------
     tableKey.removeAllElements();
     tableObject.removeAllElements();
     }

//------------------------------------------------------------------------------
//  find
//------------------------------------------------------------------------------
synchronized public int find(String key) {
     return(tableKey.find(key));
     }

//------------------------------------------------------------------------------
//  size
//------------------------------------------------------------------------------
public int size() {
     return(tableKey.size());
     }

//------------------------------------------------------------------------------
//  flip
//------------------------------------------------------------------------------
synchronized  public void flip(int item1,int item2) {
     tableKey.flip(item1,item2);
     tableObject.flip(item1,item2);
     }

//------------------------------------------------------------------------------
//  hashput without rehash
//------------------------------------------------------------------------------
public void hashput(InvisibleHashtable hash) {
     hashput(hash,false);
     }

//------------------------------------------------------------------------------
//  hashput
//------------------------------------------------------------------------------
public void hashput(InvisibleHashtable hash,boolean rehash) {
     int size = tableKey.size();

     for(int k=0;k<size;++k) {
          Object value = tableObject.elementAt(k);
          if(!(value.getClass().getName().equals("java.lang.String"))) continue;
          String key = (String)(tableKey.elementAt(k));
          if(rehash) {
               hash.set(key,InvisibleUtil.hashstring((String)(value),hash,true));
               }
          else {
               hash.set(key,(String)(value));
               }
          }
     }

//------------------------------------------------------------------------------
//  get keys
//------------------------------------------------------------------------------
public InvisibleVector getKeys() { return(tableKey); }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public InvisibleVector getTableKey() { return(tableKey); }
public InvisibleVector getTableObject() { return(tableObject); }
}
