//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceAuthenticateTest {

//------------------------------------------------------------------------------
//  main
//------------------------------------------------------------------------------
public static void main(String[] argv) {
     SpaceServer server = null;

     try {
          //--------------------------------------------------------------------
          //  parse the command line
          //--------------------------------------------------------------------
          int port = SpaceServer.defaultPort;
          InvisibleVector args = new InvisibleVector();
 
          for(int k=0;k<argv.length;++k) {
               //---------------------------------------------------------------
               //  -portno
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-portno")) {
                    port = InvisibleUtil.parseInt(argv[++k]);
                    continue;
                    }
 
               //---------------------------------------------------------------
               //  host/username/password
               //---------------------------------------------------------------
               args.set(argv[k]);
               } 
 
          String host = args.getString(0);
          String username = args.getString(1);
          String password = args.getString(2);

          //--------------------------------------------------------------------
          //  instantiate the blocks server
          //--------------------------------------------------------------------
          server = new SpaceServer(host,port);

          server.setTrace(true);

          //--------------------------------------------------------------------
          //  connect to blocks server
          //--------------------------------------------------------------------
          SpaceResponse greeting = server.connect("fr en en-us",true);

          server.free(greeting);

          //--------------------------------------------------------------------
          //  anonymous authentication
          //--------------------------------------------------------------------
          SpaceAuthenticator anonymousAuthenticator = new SpaceAuthenticatorSasl(SpaceProfile.saslAnonymous);
          anonymousAuthenticator.set("authenticator",username);

          SpaceChannel anonymousChannel = server.getChannel(anonymousAuthenticator);

          if(anonymousChannel.isError()) {
               System.err.println("Anonymous Authenticate Error: " + anonymousChannel.getErrorCode() + ":" + anonymousChannel.getErrorString());
               }
          else {
               System.err.println("Anonymous Authenticate Successful");
               }

          //--------------------------------------------------------------------
          //  sasl/otp authentication
          //--------------------------------------------------------------------
          SpaceAuthenticator otpAuthenticator = new SpaceAuthenticatorSasl(SpaceProfile.saslOtp);
          otpAuthenticator.set("authenticator",username);
          otpAuthenticator.set("passphrase",password);

          SpaceChannel otpChannel = server.getChannel(otpAuthenticator);

          if(otpChannel.isError()) {
               System.err.println("SASL/OTP Authenticate Error: " + otpChannel.getErrorCode() + ":" + otpChannel.getErrorString());
               }
          else {
               System.err.println("SASL/OTP Authenticate Successful");
               }
          }
     catch(java.lang.ArrayIndexOutOfBoundsException e) {
          System.err.println("Usage: java SpaceAuthenticator host authenticator passphrase");
          }
     catch(SpaceTimeout e) {
          System.err.println(e.toString());
          }
     catch(SpaceException e) {
          System.err.println(e.toString());
          }
     finally {
          //--------------------------------------------------------------------
          //  close the connection
          //--------------------------------------------------------------------
          if(server != null) {
               server.close();
               }
          }
     }
}
