//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.Vector;

public class SpaceBig {

//------------------------------------------------------------------------------
//  usage
//------------------------------------------------------------------------------
public static void usage() {
     System.err.println("Usage:    java SpaceBig search \"<union>...</union>\"");
     System.err.println("          java SpaceBig fetch name [element]");
     System.err.println("          java SpaceBig store subtree elementname create|update|write serial <block.xml");
     System.err.println("          java SpaceBig store subtree elementname delete serial");
     //System.err.println("          java SpaceBig sep <stdin_sep");
     System.err.println("Options: -server hostname");
     //System.err.println("         -logfile logfilename");
     System.err.println("         -trace");
     System.err.println("         -debug");
     System.exit(1);
     }

//------------------------------------------------------------------------------
//  main
//------------------------------------------------------------------------------
public static void main(String[] argv) {
     SpaceServer server = null;

     try {
          //--------------------------------------------------------------------
          //  parse the command line
          //--------------------------------------------------------------------
          String command = null;
          String host = null;
          String authenticateUser = null;
          String authenticatePassphrase = null;
          String profile = SpaceProfile.saslAnonymous;
          String logfile = null;
          boolean trace = false;
          boolean debug = false;
          Vector args = new Vector();
          int argcount = 0;

          for(int k=0;k<argv.length;++k) {
               //---------------------------------------------------------------
               //  -server
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-server")) {
                    host = argv[++k];
                    continue;
                    }

               //---------------------------------------------------------------
               //  -authenticator or -username
               //---------------------------------------------------------------
               if((argv[k].equalsIgnoreCase("-authenticator")) || (argv[k].equalsIgnoreCase("-username"))) {
                    authenticateUser = argv[++k];
                    continue;
                    }

               //---------------------------------------------------------------
               //  -passphrase implying otp logon
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-passphrase")) {
                    authenticatePassphrase = argv[++k];
                    profile = SpaceProfile.saslOtp;
                    continue;
                    }

               //---------------------------------------------------------------
               //  -logfile
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-logfile")) {
                    logfile = argv[++k];
                    continue;
                    }

               //---------------------------------------------------------------
               //  -trace
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-trace")) {
                    trace = true;
                    continue;
                    }

               //---------------------------------------------------------------
               //  -debug
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-debug")) {
                    debug = true;
                    continue;
                    }

               //---------------------------------------------------------------
               //  first arg is command
               //---------------------------------------------------------------
               if(argcount == 0) {
                    command = argv[k];
                    }
               else {
                    args.addElement(argv[k]);
                    }

               ++argcount;
               }

          //--------------------------------------------------------------------
          //  get whatever is available on standard input
          //--------------------------------------------------------------------
          StringBuffer inputContent = new StringBuffer();
          int inchar;

          BufferedReader input = new BufferedReader(new InputStreamReader(System.in));

          try {
               if(input.ready()) {
                    while((inchar = input.read()) != -1) {
                         inputContent.append((char)(inchar));
                         }
                    }
               }
          catch(IOException e) {
               }

          //--------------------------------------------------------------------
          //  instantiate the blocks server
          //--------------------------------------------------------------------
          if(host == null) usage();

          server = new SpaceServer(host);

          server.setTrace(trace);
          server.setDebug(debug);

          //--------------------------------------------------------------------
          //  connect to blocks server
          //--------------------------------------------------------------------
          SpaceResponse greeting = server.connect(true);

          server.free(greeting);

          //--------------------------------------------------------------------
          //  authentication
          //--------------------------------------------------------------------
          SpaceAuthenticator authenticator = new SpaceAuthenticatorSasl(profile);
          authenticator.set("authenticator",authenticateUser);
          authenticator.set("passphrase",authenticatePassphrase);

          SpaceChannel channel = server.getChannel(authenticator);

          if(channel.isError()) {
               System.err.println("Authenticate Error: " + channel.getErrorCode());
               return;
               }

          //--------------------------------------------------------------------
          //  start an sep channel
          //--------------------------------------------------------------------
          SpaceChannel sep = server.getChannel(SpaceProfile.sep);

          SpaceResponse response = sep.start();

          if(response.isError()) {
               System.err.println("Unable TO Setup SEP Channel" + sep.getErrorCode());
               return;
               }

          //--------------------------------------------------------------------
          //  sep
          //--------------------------------------------------------------------
          if(command.equalsIgnoreCase("sep")) {
               if(args.size() != 0) usage();

               response = sep.request(inputContent.toString(),true);

               if(response.isError()) {
                    System.err.println(response.getErrorString());
                    }
               else {
                    System.out.println(response.toString());
                    }

               return;
               }

          //--------------------------------------------------------------------
          //  search
          //--------------------------------------------------------------------
          if(command.equalsIgnoreCase("search")) {
               if(args.size() != 1) usage();

               StringBuffer request = new StringBuffer();
               request.append("<fetch>");
               request.append((String)(args.elementAt(0)));
               request.append("</fetch>");
               response = sep.request(request.toString(),true);

               if(response.isError()) {
                    System.err.println(response.getErrorString());
                    }
               else {
                    response.parse();

                    String names[] = response.getNames();

                    for(int k=0;k<names.length;++k) {
                         System.out.println(names[k]);
                         }
                    }

               return;
               }

          //--------------------------------------------------------------------
          //  fetch
          //--------------------------------------------------------------------
          if(command.equalsIgnoreCase("fetch")) {
               if((args.size() < 1) || (args.size() > 2)) usage();

               String name = (String)(args.elementAt(0));
               String element = null;
               if(args.size() == 2) {
                    element = (String)(args.elementAt(1));
                    }

               response = sep.request(SpaceSep.fetch(name,element),true);

               if(response.isError()) {
                    System.err.println(response.getErrorString());
                    }
               else {
                    System.out.println(response.toString());
                    }

               return;
               }

          //--------------------------------------------------------------------
          //  store
          //--------------------------------------------------------------------
          if(command.equalsIgnoreCase("store")) {
               if((args.size() < 1) || (args.size() > 4)) usage();

               String subtree = (String)(args.elementAt(0));
               String name = null;
               String action = null;
               String serial = null;
               if(args.size() >= 2) {
                    name = (String)(args.elementAt(1));
                    }
               if(args.size() >= 3) {
                    action = (String)(args.elementAt(2));
                    }
               if(args.size() >= 4) {
                    serial = (String)(args.elementAt(3));
                    }

               //---------------------------------------------------------------
               //  lock the subtree
               //---------------------------------------------------------------
               response = sep.request(SpaceSep.lock(subtree),true);

               if(response.isError()) {
                    System.err.println(response.getErrorString());
                    return;
                    }
               else {
                    System.out.println(response.toString());
                    }

               int prevno = response.getReqno();

               //---------------------------------------------------------------
               //  store
               //---------------------------------------------------------------
               response = sep.request(SpaceSep.store(name,action,serial,inputContent.toString()),true);

               if(response.isError()) {
                    System.err.println(response.getErrorString());
                    }
               else {
                    System.out.println(response.toString());
                    }

               //---------------------------------------------------------------
               //  release the subtree
               //---------------------------------------------------------------
               response = sep.request(SpaceSep.release(prevno),true);

               if(response.isError()) {
                    System.err.println(response.getErrorString());
                    }
               else {
                    System.out.println(response.toString());
                    }
               }
          }
     catch(NullPointerException e) {
          usage();
          }
     catch(SpaceTimeout e) {
          System.err.println(e.toString());
          }
     catch(SpaceException e) {
          System.err.println(e.toString());
          }
     finally {
          //--------------------------------------------------------------------
          //  close the connection
          //--------------------------------------------------------------------
          if(server != null) {
               server.close();
               }
          }
     }
}
