//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceExample {

//------------------------------------------------------------------------------
//  main
//------------------------------------------------------------------------------
public static void main(String[] argv) {
     SpaceServer server = null;

     try {
          //--------------------------------------------------------------------
          //  set blocks server host
          //--------------------------------------------------------------------
          if(argv.length != 1) {
               System.err.println("Usage: java SpaceExample hostname");
               return;
               }

          //--------------------------------------------------------------------
          //  instantiate the blocks server
          //--------------------------------------------------------------------
          server = new SpaceServer(argv[0]);

          server.setTrace(true);

          //--------------------------------------------------------------------
          //  connect to blocks server
          //--------------------------------------------------------------------
          SpaceResponse greeting = server.connect(true);

          //--------------------------------------------------------------------
          //  show valid profiles
          //--------------------------------------------------------------------
          String[] profiles = greeting.getProfiles();
          for(int k=0;k<profiles.length;++k) {
               System.err.println("Profile: " + profiles[k]);
               }

          //--------------------------------------------------------------------
          //  show localization
          //--------------------------------------------------------------------
          System.err.println("Localize: " + greeting.getLocalize());

          //--------------------------------------------------------------------
          //  show features
          //--------------------------------------------------------------------
          System.err.println("Features: " + greeting.getFeatures());

          //--------------------------------------------------------------------
          //  free the greeting
          //--------------------------------------------------------------------
          server.free(greeting);

          //--------------------------------------------------------------------
          //  start a non-authenticated channel
          //--------------------------------------------------------------------
          SpaceChannel channel = server.getChannel(SpaceProfile.sep);
          SpaceResponse response = channel.start(false);

          //--------------------------------------------------------------------
          //  build an sep fetch request while waiting for start response
          //--------------------------------------------------------------------
          StringBuffer fetch = new StringBuffer();

          fetch.append("<fetch>\r\n");
          fetch.append("<union>\r\n");
          fetch.append("<intersect>\r\n");
          fetch.append("<compare subtree='doc.rfc' operator='contains' ");
          fetch.append("caseSensitive='false'>\r\n");
          fetch.append("<path><element property='email' /></path>\r\n");
          fetch.append("<value>mrose@</value>\r\n");
          fetch.append("</compare>\r\n");
          fetch.append("</intersect>\r\n");
          fetch.append("</union>\r\n");
          fetch.append("</fetch>\r\n");

          //--------------------------------------------------------------------
          //  wait on the channel start and see if error
          //--------------------------------------------------------------------
          channel.wait(response);

          if(response.isError()) {
               System.err.println("Start Error: " + response.getErrorCode());
               return;
               }

          //--------------------------------------------------------------------
          //  do the request
          //--------------------------------------------------------------------
          response = channel.request(fetch.toString(),true);

          if(response.isError()) {
               System.err.println("Request Error: " + response.getErrorCode());
               return;
               }

          //--------------------------------------------------------------------
          //  show the response on standard output
          //--------------------------------------------------------------------
          System.out.println(response.toString());

          //--------------------------------------------------------------------
          //  free the response
          //--------------------------------------------------------------------
          channel.free(response);
          }
     catch(SpaceTimeout e) {
          System.err.println(e.toString());
          }
     catch(SpaceException e) {
          System.err.println(e.toString());
          }
     finally {
          //--------------------------------------------------------------------
          //  always close the server
          //--------------------------------------------------------------------
          if(server != null) {
               server.close();
               }
          }
     }
}
