//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpacePing {

//------------------------------------------------------------------------------
//  main
//------------------------------------------------------------------------------
public static void main(String[] argv) {
     SpaceServer server = null;

     try {
          //--------------------------------------------------------------------
          //  parse the command line
          //--------------------------------------------------------------------
          String host = null;
          int port = SpaceServer.defaultPort;
 
          for(int k=0;k<argv.length;++k) {
               //---------------------------------------------------------------
               //  -portno
               //---------------------------------------------------------------
               if(argv[k].equalsIgnoreCase("-portno")) {
                    port = InvisibleUtil.parseInt(argv[++k]);
                    continue;
                    }
 
               //---------------------------------------------------------------
               //  host
               //---------------------------------------------------------------
               host = argv[k];
               } 
 
          //--------------------------------------------------------------------
          //  instantiate the blocks server
          //--------------------------------------------------------------------
          server = new SpaceServer(host,port);

          //--------------------------------------------------------------------
          //  connect to blocks server
          //--------------------------------------------------------------------
          SpaceResponse greeting = server.connect(true);

          server.free(greeting);

          //--------------------------------------------------------------------
          //  start an echo channel
          //--------------------------------------------------------------------
          SpaceChannel channel = server.getChannel(SpaceProfile.nullEcho);
          SpaceResponse echoStart = server.start(channel,true);
          SpaceResponse echoResponse = server.request(channel,"Ping",true);

          //--------------------------------------------------------------------
          //  start a sink channel
          //--------------------------------------------------------------------
          SpaceChannel sinkChannel = server.getChannel(SpaceProfile.nullSink);
          SpaceResponse sinkStart = server.start(sinkChannel,true);
          SpaceResponse sinkResponse = server.request(sinkChannel,"Gone",true);

          //--------------------------------------------------------------------
          //  show server responses
          //--------------------------------------------------------------------
          System.err.println("Echo: " + echoResponse.toString());
          System.err.println("Sink: " + sinkResponse.toString());

          //--------------------------------------------------------------------
          //  free all responses
          //--------------------------------------------------------------------
          server.free(echoStart);
          server.free(echoResponse);
          server.free(sinkStart);
          server.free(sinkResponse);
          }
     catch(NullPointerException e) {
          System.err.println("java SpacePing [-portno port] host");
          }
     catch(SpaceTimeout e) {
          System.err.println(e.toString());
          }
     catch(SpaceException e) {
          System.err.println(e.toString());
          }
     finally {
          //--------------------------------------------------------------------
          //  close the connection
          //--------------------------------------------------------------------
          if(server != null) {
               server.close();
               }
          }
     }
}
