//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.io.InputStream;
import java.io.IOException;
import java.util.Hashtable;
import org.apache.xerces.parsers.*;
import org.w3c.dom.*;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.dom.ElementImpl;
import org.apache.xerces.dom.NamedNodeMapImpl;
import org.xml.sax.InputSource;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class SpaceTest {

//------------------------------------------------------------------------------
//  main
//------------------------------------------------------------------------------
public static void main(String[] argv) {
     SpaceServer server = null;

     try {
          //--------------------------------------------------------------------
          //  instantiate the blocks server
          //--------------------------------------------------------------------
          server = new SpaceServer();

          server.setTrace(true);

          //--------------------------------------------------------------------
          //  set blocks server host
          //--------------------------------------------------------------------
          if(argv.length != 1) {
               System.err.println("Usage: java SpaceTest hostname");
               return;
               }

          server.setHost(argv[0]);

          //--------------------------------------------------------------------
          //  connect to blocks server
          //--------------------------------------------------------------------
          SpaceResponse greetingResponse = server.connect("fr en en-us",true);

          String[] profiles = greetingResponse.getProfiles();
          for(int k=0;k<profiles.length;++k) {
               System.err.println("Profile: " + profiles[k]);
               }

          //--------------------------------------------------------------------
          //  start a bunch of channels
          //--------------------------------------------------------------------
          SpaceChannel channel1 = server.getChannel(SpaceProfile.sep);
          SpaceChannel channel2 = server.getChannel(SpaceProfile.sep);
          SpaceChannel channel3 = server.getChannel(SpaceProfile.sep);
          SpaceChannel channel4 = server.getChannel(SpaceProfile.sep);
          SpaceChannel channel5 = server.getChannel(SpaceProfile.sep);
          SpaceChannel channel6 = server.getChannel(SpaceProfile.sep);
          SpaceChannel channel7 = server.getChannel(SpaceProfile.sep);
          SpaceChannel channel8 = server.getChannel(SpaceProfile.sep);

          SpaceResponse start1 = channel1.start(false);
          SpaceResponse start2 = channel2.start(false);
          SpaceResponse start3 = channel3.start(false);
          SpaceResponse start4 = channel4.start(false);
          SpaceResponse start5 = channel5.start(false);
          SpaceResponse start6 = channel6.start(false);
          SpaceResponse start7 = channel7.start(false);
          SpaceResponse start8 = channel8.start(false);

          //--------------------------------------------------------------------
          //  fetch short results
          //--------------------------------------------------------------------
          StringBuffer fetchShortResults = new StringBuffer();

          fetchShortResults.append("<fetch>\r\n");
          fetchShortResults.append("<union>\r\n");
          fetchShortResults.append("<intersect>\r\n");
          fetchShortResults.append("<compare subtree='doc.rfc' operator='contains' caseSensitive='false'>\r\n");
          fetchShortResults.append("<path><element property='email' /></path>\r\n");
          fetchShortResults.append("<value>kzm@</value>\r\n");
          //fetchShortResults.append("<value>mrose@</value>\r\n");
          fetchShortResults.append("</compare>\r\n");
          fetchShortResults.append("</intersect>\r\n");
          fetchShortResults.append("</union>\r\n");
          fetchShortResults.append("</fetch>\r\n");

          //--------------------------------------------------------------------
          //  fetch long results
          //--------------------------------------------------------------------
          StringBuffer fetchLongResults = new StringBuffer();

          fetchLongResults.append("<fetch>\r\n");
          fetchLongResults.append("<union>\r\n");
          fetchLongResults.append("<intersect>\r\n");
          fetchLongResults.append("<compare subtree='doc.edgar' operator='contains' caseSensitive='false'>\r\n");
          fetchLongResults.append("<path><element property='state.of.incorporation' /></path>\r\n");
          fetchLongResults.append("<value>IN</value>\r\n");
          fetchLongResults.append("</compare>\r\n");
          fetchLongResults.append("</intersect>\r\n");
          fetchLongResults.append("</union>\r\n");
          fetchLongResults.append("</fetch>\r\n");

          //--------------------------------------------------------------------
          //  test wait timeout
          //--------------------------------------------------------------------
          SpaceResponse responseWait = channel3.request(fetchLongResults.toString(),false);

          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Now wait until completed......complete: " + responseWait.isComplete());
          channel3.wait(responseWait);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());

          //--------------------------------------------------------------------
          //  wait for an already completed response to show you can do it
          //--------------------------------------------------------------------
          channel3.wait(responseWait);

          //--------------------------------------------------------------------
          //  free the response
          //--------------------------------------------------------------------
          channel3.free(responseWait);

          //--------------------------------------------------------------------
          //  do a bunch of requests on different channels
          //--------------------------------------------------------------------
          SpaceResponse response1 = channel1.request(fetchShortResults.toString(),false);
          SpaceResponse response2 = channel2.request(fetchShortResults.toString(),false);
          SpaceResponse response3 = channel3.request(fetchShortResults.toString(),false);
          SpaceResponse response4 = channel4.request(fetchShortResults.toString(),false);
          SpaceResponse response5 = channel5.request(fetchShortResults.toString(),false);
          SpaceResponse response6 = channel6.request(fetchShortResults.toString(),false);
          SpaceResponse response7 = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response8 = channel8.request(fetchShortResults.toString(),false);

          //--------------------------------------------------------------------
          //  do a bunch of requests on the same channel
          //--------------------------------------------------------------------
          SpaceResponse response7a = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response7b = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response7c = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response7d = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response7e = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response7f = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response7g = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response7h = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response7i = channel7.request(fetchShortResults.toString(),false);
          SpaceResponse response7j = channel7.request(fetchShortResults.toString(),false);

          //--------------------------------------------------------------------
          //  wait on all the responses to flush the response cache
          //--------------------------------------------------------------------
          channel7.wait(response7a);
          channel7.wait(response7b);
          channel7.wait(response7c);
          channel7.wait(response7d);
          channel7.wait(response7e);
          channel7.wait(response7f);
          channel7.wait(response7g);
          channel7.wait(response7h);
          channel7.wait(response7i);
          channel7.wait(response7j);

          channel8.wait(response8);
          channel7.wait(response7);
          channel6.wait(response6);
          channel5.wait(response5);
          channel4.wait(response4);
          channel3.wait(response3);
          channel2.wait(response2);
          channel1.wait(response1);

          //--------------------------------------------------------------------
          //  flush out the start responses
          //--------------------------------------------------------------------
          channel1.wait(start1);
          channel2.wait(start2);
          channel3.wait(start3);
          channel4.wait(start4);
          channel5.wait(start5);
          channel6.wait(start6);
          channel7.wait(start7);
          channel8.wait(start8);

          //--------------------------------------------------------------------
          //  get a response and parse it
          //--------------------------------------------------------------------
          SpaceResponse responseParse = channel7.request(fetchLongResults.toString(),false);

          channel7.wait(responseParse);

          InputStream in = responseParse.getInputStream();

          System.err.println("XML Stream Size: " + in.available());

          DOMParser parser = new DOMParser();
          parser.setFeature("http://apache.org/xml/features/dom/defer-node-expansion", true);
          parser.setFeature("http://apache.org/xml/features/continue-after-fatal-error", true);
          parser.parse(new InputSource(in));
          DocumentImpl document = (DocumentImpl)parser.getDocument();
 
          //--------------------------------------------------------------------
          //  show
          //--------------------------------------------------------------------
          Element root = document.getDocumentElement();

          System.err.println("XML Root Name: " + root.getTagName());

          NodeList list = root.getElementsByTagName("answers");

          for(int k=0;k<list.getLength();++k) {
               Node node = list.item(k);
               NodeList children = node.getChildNodes();
               for(int j=0;j<children.getLength();++j) {
                    Node child = children.item(j);
                    if(child.getNodeName().startsWith("#")) continue;
                    System.err.println("Node: " + node.getNodeName() + "." + child.getNodeName());
                    }
               }

          //--------------------------------------------------------------------
          //  force the server to timeout by waiting more than 10 minutes
          //--------------------------------------------------------------------
          System.err.println("TEST: Starting server timeout test...");
          System.err.println("TEST: 15 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 14 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 13 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 12 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 11 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 10 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 9 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 8 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 7 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 6 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 5 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 4 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 3 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 2 minutes to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 1 minute to go....");
          InvisibleUtil.sleepSeconds(60);
          System.err.println("TEST: 0 minutes to go....");
          System.err.println("TEST: waking up...");

          //--------------------------------------------------------------------
          //  get a simple result from the timed out connection
          //--------------------------------------------------------------------
          SpaceResponse responseTimeout = channel1.request(fetchShortResults.toString(),false);
          System.err.println("TEST: Pass: Timeout Request");
          channel1.wait(responseTimeout);
          System.err.println("TEST: Pass: Timeout Wait");
          }
     catch(SpaceTimeout e) {
          System.err.println(e.toString());
          }
     catch(SpaceException e) {
          System.err.println(InvisibleUtil.getStackTrace(e));
          }
     catch(org.xml.sax.SAXException e) {
          System.err.println(InvisibleUtil.getStackTrace(e));
          }
     catch(IOException e) {
          System.err.println(InvisibleUtil.getStackTrace(e));
          }
     finally {
          System.err.println("TEST: Shutdown");

          if(server != null) {
               server.close();
               }
          }
     }
}
