//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceTest2 {

//------------------------------------------------------------------------------
//  main
//------------------------------------------------------------------------------
public static void main(String[] argv) {
     SpaceServer server = null;

     try {
          //--------------------------------------------------------------------
          //  instantiate the blocks server
          //--------------------------------------------------------------------
          server = new SpaceServer();

          server.setTrace(true);


          //--------------------------------------------------------------------
          //  set blocks server host
          //--------------------------------------------------------------------
          if(argv.length != 1) {
               System.err.println("Usage: java SpaceTest hostname");
               return;
               }

          server.setHost(argv[0]);

          //--------------------------------------------------------------------
          //  connect to blocks server
          //--------------------------------------------------------------------
          SpaceResponse greetingResponse = server.connect(true);

          String[] profiles = greetingResponse.getProfiles();
          for(int k=0;k<profiles.length;++k) {
               System.err.println("Profile: " + profiles[k]);
               }

          //--------------------------------------------------------------------
          //  start a bunch of channels
          //--------------------------------------------------------------------
          int channelCount = 3;

          SpaceChannel[] channel = new SpaceChannel[channelCount];
          SpaceResponse[] response = new SpaceResponse[channelCount];

          for(int k=0;k<channelCount;++k) {
               channel[k] = server.getChannel(SpaceProfile.sep);
               response[k] = server.start(channel[k],false);
               }

          //--------------------------------------------------------------------
          //  fetch short results
          //--------------------------------------------------------------------
          StringBuffer fetchShortResults = new StringBuffer();

          fetchShortResults.append("<fetch>\r\n");
          fetchShortResults.append("<union>\r\n");
          fetchShortResults.append("<intersect>\r\n");
          fetchShortResults.append("<compare subtree='doc.rfc' operator='contains' caseSensitive='false'>\r\n");
          fetchShortResults.append("<path><element property='email' /></path>\r\n");
          fetchShortResults.append("<value>kzm@</value>\r\n");
          //fetchShortResults.append("<value>mrose@</value>\r\n");
          fetchShortResults.append("</compare>\r\n");
          fetchShortResults.append("</intersect>\r\n");
          fetchShortResults.append("</union>\r\n");
          fetchShortResults.append("</fetch>\r\n");

          String fetchShort = fetchShortResults.toString();

          //--------------------------------------------------------------------
          //  fetch long results
          //--------------------------------------------------------------------
          StringBuffer fetchLongResults = new StringBuffer();

          fetchLongResults.append("<fetch>\r\n");
          fetchLongResults.append("<union>\r\n");
          fetchLongResults.append("<intersect>\r\n");
          fetchLongResults.append("<compare subtree='doc.edgar' operator='contains' caseSensitive='false'>\r\n");
          fetchLongResults.append("<path><element property='state.of.incorporation' /></path>\r\n");
          fetchLongResults.append("<value>IN</value>\r\n");
          fetchLongResults.append("</compare>\r\n");
          fetchLongResults.append("</intersect>\r\n");
          fetchLongResults.append("</union>\r\n");
          fetchLongResults.append("</fetch>\r\n");

          String fetchLong = fetchLongResults.toString();

          //--------------------------------------------------------------------
          //  test wait timeout
          //--------------------------------------------------------------------
          SpaceResponse responseWait = server.request(channel[0],fetchLongResults.toString(),false);

          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());
          server.wait(responseWait,2000);
          System.err.println("TEST: Now wait until completed......complete: " + responseWait.isComplete());
          server.wait(responseWait);
          System.err.println("TEST: Query Started......complete: " + responseWait.isComplete());

          //--------------------------------------------------------------------
          //  wait for an already completed response
          //--------------------------------------------------------------------
          server.wait(responseWait);

          //--------------------------------------------------------------------
          //  free the response
          //--------------------------------------------------------------------
          server.free(responseWait);

          //--------------------------------------------------------------------
          //  clear out all the old start responses
          //--------------------------------------------------------------------
          for(int k=0;k<channelCount;++k) {
               server.wait(response[k]);
               }

          //--------------------------------------------------------------------
          //  do a random request on each channel
          //--------------------------------------------------------------------
          for(int k=0;k<channelCount;++k) {
               String s = ((System.currentTimeMillis()&1)==0)?fetchShort:fetchLong;
               response[k] = server.request(channel[k],s,false);
               }

          //--------------------------------------------------------------------
          //  clean out the waits
          //--------------------------------------------------------------------
          for(int k=0;k<channelCount;++k) {
               server.wait(response[k]);
               }
          }
     catch(SpaceTimeout e) {
          System.err.println(e.toString());
          }
     catch(SpaceException e) {
          System.err.println(InvisibleUtil.getStackTrace(e));
          }
     finally {
          System.err.println("TEST: Shutdown");

          if(server != null) {
               server.close();
               }
          }
     }
}
