//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceAuthenticatorSasl extends SpaceAuthenticatorCommon implements SpaceAuthenticator {

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceAuthenticatorSasl(String profileArg) {
     super(profileArg);
     }

//------------------------------------------------------------------------------
//  get authentication request
//------------------------------------------------------------------------------
public String getRequest() throws SpaceException {
     //-------------------------------------------------------------------------
     //  get the authenticator
     //-------------------------------------------------------------------------
     String authenticator = settings.getString("authenticator");

     if(authenticator != null) {
          return(getRequest(authenticator));
          }

     return("");
     }

//------------------------------------------------------------------------------
//  get authentication response
//------------------------------------------------------------------------------
public String getResponse(SpaceResponse challengeResponse) throws SpaceException {
     //-------------------------------------------------------------------------
     //  get the pass phrase
     //-------------------------------------------------------------------------
     String passPhrase = settings.getString("passphrase");

     //-------------------------------------------------------------------------
     //  get the challenge
     //-------------------------------------------------------------------------
     String challenge = challengeResponse.getChallengeString();

     if((passPhrase != null) && (passPhrase.length() > 0) && (challenge != null)) {
          //--------------------------------------------------------------------
          //  decode/parse the challenge [otp-sha1, 9993, rwcsqa0130117, ext]
          //--------------------------------------------------------------------
          InvisibleVector v = InvisibleUtil.parse(InvisibleBase64.decode(challenge)," ");

          //--------------------------------------------------------------------
          //  build the response
          //--------------------------------------------------------------------
          String response = null;

          try {
               String algorithm = v.getString(0);
               String seed = v.getString(2);
               int count = v.getInt(1);

               InvisibleOtp otp = new InvisibleOtp(algorithm,passPhrase,seed,count);

               response = otp.get("word");
               }
          catch(InvisibleException e) {
               throw new SpaceException("SpaceProfile: " + e.toString());
               }
          catch(ArrayIndexOutOfBoundsException e) {
               throw new SpaceException("SpaceProfile: Unrecognized Challenge: " + challenge);
               }
          catch(NullPointerException e) {
               throw new SpaceException("SpaceProfile: Unrecognized Challenge: " + challenge);
               }

          //--------------------------------------------------------------------
          //  send the response
          //--------------------------------------------------------------------
          StringBuffer sb = new StringBuffer();

          sb.append("<blob>");
          sb.append(InvisibleBase64.encode(response));
          sb.append("</blob>");

          return(sb.toString());
          }

     return("");
     }
}
