//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceCache extends InvisibleHashtable {

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceCache() {
     super();
     }

//------------------------------------------------------------------------------
//  set a response
//------------------------------------------------------------------------------
synchronized public void set(int serial,SpacePacket response) {
     //-------------------------------------------------------------------------
     //  convert serial to string for hashtable
     //-------------------------------------------------------------------------
     String serialKey = String.valueOf(serial);

     //-------------------------------------------------------------------------
     //  set
     //-------------------------------------------------------------------------
     super.set(serialKey,response);
     }

//------------------------------------------------------------------------------
//  get a response
//------------------------------------------------------------------------------
synchronized public SpaceResponse get(int serial) {
     //-------------------------------------------------------------------------
     //  convert serial to string for hashtable
     //-------------------------------------------------------------------------
     String serialKey = String.valueOf(serial);

     //-------------------------------------------------------------------------
     //  see if serial already exists
     //-------------------------------------------------------------------------
     SpaceResponse response;

     if((response = (SpaceResponse)(super.get(serialKey))) != null) {
          //--------------------------------------------------------------------
          //  response found
          //--------------------------------------------------------------------
          return(response);
          }

     //-------------------------------------------------------------------------
     //  response not found
     //-------------------------------------------------------------------------
     return(null);
     }

//------------------------------------------------------------------------------
//  remove a response
//------------------------------------------------------------------------------
synchronized public void remove(int serial,boolean freeResponse) {
     //-------------------------------------------------------------------------
     //  convert serial to string for hashtable
     //-------------------------------------------------------------------------
     String serialKey = String.valueOf(serial);

     //-------------------------------------------------------------------------
     //  free the response if appropriate
     //-------------------------------------------------------------------------
     if(freeResponse) {
          SpaceResponse response;

          if((response = (SpaceResponse)(super.get(serialKey))) != null) {
               response.free();
               }
          }

     //-------------------------------------------------------------------------
     //  remove
     //-------------------------------------------------------------------------
     super.remove(serialKey);
     }

//------------------------------------------------------------------------------
//  set all responses as available and in error for lost connection
//------------------------------------------------------------------------------
synchronized public void setErrorAll() {
     InvisibleVector v = getVector();

     int size = v.size();

     for(int k=0;k<size;++k) {
          SpaceResponse response = (SpaceResponse)(v.get(k));
          if(response.isIncomplete()) {
               response.setIncomplete(false);
               response.setError(true);
               }
          }
     }

//------------------------------------------------------------------------------
//  trace to stderr
//------------------------------------------------------------------------------
synchronized public void trace() {
     System.err.println("-------------------------------------------------------------------------");
     System.err.println(": Type :  ID  : Channel : Serial :  Length  : Frames : Complete : Error :");
     System.err.println("-------------------------------------------------------------------------");
     InvisibleVector v = getVector();

     int size = v.size();

     for(int k=0;k<size;++k) {
          SpacePacket response = (SpacePacket)(v.get(k));
          StringBuffer sb = new StringBuffer();

          sb.append(": ");
          try {
               if(response.isRequest()) {
                    sb.append("REQ  : ");
                    }
               else {
                    sb.append("RSP  : ");
                    }
               }
          catch(NullPointerException e) {
               sb.append("RSP  : "); //no header yet
               }
          sb.append(InvisibleUtil.fill(k,4,' '));
          sb.append(" : ");
          sb.append(InvisibleUtil.fill(response.getChannelElement().getChannel(),7,' '));
          sb.append(" : ");
          sb.append(InvisibleUtil.fill(response.getSerial(),6,' '));
          sb.append(" : ");
          sb.append(InvisibleUtil.pad(response.length(),8,' '));
          sb.append(" : ");
          sb.append(InvisibleUtil.fill(response.getFrameCount(),6,' '));
          sb.append(" : ");
          sb.append(InvisibleUtil.fill(response.isComplete(),8,' '));
          sb.append(" : ");
          sb.append(InvisibleUtil.fill(response.isError(),5,' '));
          sb.append(" :");

          System.err.println(sb.toString());
          }

     System.err.println("-------------------------------------------------------------------------");
     }
}
