//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceChannel {

private SpaceServer                     server;
private long                            seqno;
private int                             channel;
private String                          mime;
private String                          profile;
private InvisibleHashtable              parameter;

private boolean                         started;
private boolean                         error;
private int                             errorCode;
private String                          errorCodeString;
private String                          errorString;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceChannel(int channelArg) throws SpaceException {
     init(channelArg);
     }

//------------------------------------------------------------------------------
//  init
//------------------------------------------------------------------------------
private void init(int channelArg) throws SpaceException {
     seqno = 0L;
     channel = channelArg;
     mime = null;

     if(channel == 0) {
          profile = SpaceProfile.sep;
          }
     else {
          profile = SpaceProfile.sep;
          }

     if((channel != 0) && ((channel & 1) != 1)) {
          throw new SpaceException("SpaceChannel: Channel Must Be Odd Number");
          }

     parameter = null;

     started = false;
     error = false;
     errorCode = 0;
     errorCodeString = null;
     errorString = null;
     }

//------------------------------------------------------------------------------
//  start
//------------------------------------------------------------------------------
public SpaceResponse start() throws SpaceTimeout,SpaceException {
     isServerDefined();
     return(server.start(this));
     }

public SpaceResponse start(boolean wait) throws SpaceTimeout,SpaceException {
     isServerDefined();
     return(server.start(this,wait));
     }

//------------------------------------------------------------------------------
//  respond
//------------------------------------------------------------------------------
public SpaceResponse respond(String request) throws SpaceTimeout,SpaceException {
     isServerDefined();
     return(server.respond(this,request));
     }

public SpaceResponse respond(String request,boolean wait) throws SpaceTimeout,SpaceException {
     isServerDefined();
     return(server.respond(this,request,wait));
     }

//------------------------------------------------------------------------------
//  request
//------------------------------------------------------------------------------
public SpaceResponse request(String request) throws SpaceTimeout,SpaceException {
     isServerDefined();
     return(server.request(this,request));
     }

public SpaceResponse request(String request,boolean wait) throws SpaceTimeout,SpaceException {
     isServerDefined();
     return(server.request(this,request,wait));
     }

//------------------------------------------------------------------------------
//  wait
//------------------------------------------------------------------------------
public SpaceResponse wait(SpaceResponse response) throws SpaceException {
     isServerDefined();
     return(server.wait(response));
     }   
 
public SpaceResponse wait(SpaceResponse response,long timeout) throws SpaceException {
     isServerDefined();
     return(server.wait(response,timeout));
     }
 
 
//------------------------------------------------------------------------------
//  response free 
//------------------------------------------------------------------------------ 
synchronized public void free(SpaceResponse response) throws SpaceException {
     isServerDefined();
     server.free(response);
     }

//------------------------------------------------------------------------------
//  verify server defined
//------------------------------------------------------------------------------
private void isServerDefined() throws SpaceException {
     if(server == null) {
          throw new SpaceException("SpaceChannel: No Connection");
          }
     }

//------------------------------------------------------------------------------
//  reset
//------------------------------------------------------------------------------
public void reset() throws SpaceException {
     init(channel);
     }

//------------------------------------------------------------------------------
//  set next sequence--increment by payload size
//------------------------------------------------------------------------------
public long setNextSeqno(int payloadSize) {
     seqno += (long)(payloadSize);
     return(seqno);
     }

//------------------------------------------------------------------------------
//  parameter handling
//------------------------------------------------------------------------------
synchronized public void setParameter(String key,String value) {
     if(parameter == null) parameter = new InvisibleHashtable();
     parameter.set(key,value);
     }

public String getParameter(String key) {
     if(parameter == null) return(null);
     return((String)(parameter.get(key)));
     }

//------------------------------------------------------------------------------
//  conditionally allow setting based on activity
//------------------------------------------------------------------------------
private void isStartedException() throws SpaceException {
     if(started) {
          throw new SpaceException("SpaceChannel: Channel Already Started");
          }
     }

//------------------------------------------------------------------------------
//  set errors from parsed response
//------------------------------------------------------------------------------
public void setError(SpaceResponse response) throws SpaceException {
     setError(true);
     setErrorCode(response.getErrorCode());
     setErrorCodeString(response.getErrorCodeString());
     setErrorString(response.getErrorString());
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public boolean isStarted() { return(started); }
public boolean isNotStarted() { return(!(started)); }
public boolean isError() { return(error); }

public void setServer(SpaceServer value) throws SpaceException { isStartedException(); server = value; }
public void setSeqno(long value) throws SpaceException { isStartedException(); seqno = value; }
public void setChannel(int value) throws SpaceException { isStartedException(); channel = value; }
public void setMime(String value) { mime = value; } //INCOMPLETE: needs to move to SpacePacket
public void setProfile(String value) throws SpaceException { isStartedException(); profile = value; }
public void setStarted(boolean value) throws SpaceException { isStartedException(); started = value; }
public void setError(boolean value) { error = value; }
public void setErrorCode(int value) { errorCode = value; }
public void setErrorCodeString(String value) { errorCodeString = value; }
public void setErrorString(String value) { errorString = value; }

public long getSeqno() { return(seqno); }
public int getChannel() { return(channel); }
public String getMime() { return(mime); }
public String getProfile() { return(profile); }
public boolean getStarted() { return(started); }
public boolean getError() { return(error); }
public int getErrorCode() { return(errorCode); }
public String getErrorCodeString() { return(errorCodeString); }
public String getErrorString() { return(errorString); }
}
