//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;

public class SpaceConfigCache extends InvisibleHashtable {
 
int                                     defaultPort;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceConfigCache() {
     super();

     defaultPort = 0;
     }

//------------------------------------------------------------------------------
//  get a configuration by port number
//------------------------------------------------------------------------------
public SpaceConfig get(int port) {
     return((SpaceConfig)(get(String.valueOf(port))));
     }

//------------------------------------------------------------------------------
//  get default configuration
//------------------------------------------------------------------------------
public SpaceConfig getDefault() {
     return(get(defaultPort));
     }

//------------------------------------------------------------------------------
//  load
//------------------------------------------------------------------------------
public void load(String pathname) throws SpaceException {
     //-------------------------------------------------------------------------
     //  load the config file contents
     //-------------------------------------------------------------------------
     try {
          //--------------------------------------------------------------------
          //  open the input stream
          //--------------------------------------------------------------------
          FileInputStream f = new FileInputStream(pathname);
          BufferedReader input = new BufferedReader(new InputStreamReader(f));

          //--------------------------------------------------------------------
          //  load the config file contents
          //--------------------------------------------------------------------
          StringBuffer sb = new StringBuffer();
          String instring;

          while((instring = input.readLine()) != null) {
               sb.append(instring);
               sb.append("\n");
               }

          //--------------------------------------------------------------------
          //  close
          //--------------------------------------------------------------------
          input.close();
          f.close();

          //--------------------------------------------------------------------
          //  parse
          //--------------------------------------------------------------------
          parse(sb);
          }
     catch(FileNotFoundException e) {
          throw new SpaceException("SpaceConfigCache: Configuration File Not Found");
          }
     catch(IOException e) {
          throw new SpaceException("SpaceConfigCache: Configuration File Read Error: " + e.toString());
          }
     }

//------------------------------------------------------------------------------
//  parse config file
//------------------------------------------------------------------------------
public void parse(StringBuffer sb) throws SpaceException {
     try {
          SpaceConfig config = null;
          SpaceProfile profile = null;
          String name = null;
          defaultPort = 0;

          InvisibleTokenizer token = new InvisibleTokenizer(sb);

          while(token.next()) {
               //---------------------------------------------------------------
               //  get element/attribute key/value
               //---------------------------------------------------------------
               String key = token.getKey();
               String value = token.getValue();

               //---------------------------------------------------------------
               //  element
               //---------------------------------------------------------------
               if(token.isElement()) {
                    //----------------------------------------------------------
                    //  ?xml
                    //----------------------------------------------------------
                    if(key.equals("?xml")) {
                         continue;
                         }

                    //----------------------------------------------------------
                    //  config
                    //----------------------------------------------------------
                    if(key.equals("config")) {
                         continue;
                         }

                    //----------------------------------------------------------
                    //  bxxd
                    //----------------------------------------------------------
                    if(key.equals("bxxd")) {
                         continue;
                         }

                    //----------------------------------------------------------
                    //  parameter
                    //----------------------------------------------------------
                    if(key.equals("parameter")) {
                         continue;
                         }

                    //----------------------------------------------------------
                    //  profile
                    //----------------------------------------------------------
                    if(key.equals("profile")) {
                         continue;
                         }

                    throw new SpaceException("SpaceConfigCache: Unknown Element: " + key);
                    }

               //---------------------------------------------------------------
               //  element close
               //---------------------------------------------------------------
               if(token.isClosingElement()) {
                    //----------------------------------------------------------
                    //  config
                    //----------------------------------------------------------
                    if(key.equals("config")) {
                         continue;
                         }

                    //----------------------------------------------------------
                    //  bxxd
                    //----------------------------------------------------------
                    if(key.equals("bxxd")) {
                         continue;
                         }

                    //----------------------------------------------------------
                    //  close current profile
                    //----------------------------------------------------------
                    if(key.equals("profile")) {
                         profile = null;
                         continue;
                         }

                    throw new SpaceException("SpaceConfigCache: Unknown Close Element: " + key);
                    }

               //---------------------------------------------------------------
               //  attribute
               //---------------------------------------------------------------
               if(token.isAttribute()) {
                    //----------------------------------------------------------
                    //  version
                    //----------------------------------------------------------
                    if(key.equals("version")) {
                         continue;
                         }

                    //----------------------------------------------------------
                    //  get/start a config
                    //----------------------------------------------------------
                    if(key.equals("port")) {
                         if((config = ((SpaceConfig)(get(value)))) == null) {
                              config = new SpaceConfig();
                              config.set(key,value);
                              set(value,config);
                              }

                         //-----------------------------------------------------
                         //  default port is first one in config file
                         //-----------------------------------------------------
                         if(defaultPort <= 0) defaultPort = InvisibleUtil.parseInt(value);
                         continue;
                         }

                    //----------------------------------------------------------
                    //  get/start a profile
                    //----------------------------------------------------------
                    if(key.equals("uri")) {
                         if(config == null) {
                              throw new SpaceException("SpaceConfigCache: No Configuration For Profile");
                              }

                         profile = config.getProfile(value);
                         continue;
                         }

                    //----------------------------------------------------------
                    //  save a module
                    //----------------------------------------------------------
                    if(key.equals("module")) {
                         if(config == null) {
                              throw new SpaceException("SpaceConfigCache: No Configuration For Module");
                              }

                         if(profile == null) {
                              throw new SpaceException("SpaceConfigCache: No Profile For Module");
                              }

                         profile.set(key,value);
                         continue;
                         }

                    //----------------------------------------------------------
                    //  save a name--name attribute must be before value
                    //----------------------------------------------------------
                    if(key.equals("name")) {
                         name = value;
                         continue;
                         }

                    //----------------------------------------------------------
                    //  set a value
                    //----------------------------------------------------------
                    if(key.equals("value")) {
                         if(config == null) {
                              throw new SpaceException("SpaceConfigCache: No Configuration For Parameter");
                              }

                         if(profile == null) {
                              //------------------------------------------------
                              //  parameter for this config
                              //------------------------------------------------
                              config.set(name,value);
                              }
                         else {
                              //------------------------------------------------
                              //  parameter for this profile
                              //------------------------------------------------
                              profile.set(name,value);
                              }

                         continue;
                         }

                    throw new SpaceException("SpaceConfigCache: Unknown Attribute: " + key);
                    }
               }
          }
     catch(InvisibleException e) {
          throw new SpaceException("SpaceConfigCache: " + e.toString());
          }
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public int getDefaultPort() { return(defaultPort); }
}
