//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.FileNotFoundException;
import java.io.IOException;

public class SpaceLog {

public final static int                 error = 0;
public final static int                 debug1 = 1;
public final static int                 debug2 = 2;
public final static int                 debug3 = 3;
public final static int                 debug4 = 4;
public final static int                 notify = 5;
public final static int                 fatal = 6;
public final static int                 info = 7;
public final static int                 stats = 8;
public final static int                 system = 9;
public final static int                 user = 10;

private static String[] levelText = {"error  ","debug1 ","debug2 ","debug3 ","debug4 ","notify ","fatal  ","info   ","stats  ","system ","user   "};

private SpaceConfig                     config;
private int                             port;
private String                          portText;

private PrintWriter                     out;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceLog() throws SpaceException {
     //-------------------------------------------------------------------------
     //  log to System.err
     //-------------------------------------------------------------------------
     config = null;
     port = 0;
     portText = "";

     out = new PrintWriter(System.err);
     }

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceLog(SpaceConfig configArg,int portArg) throws SpaceException {
     //-------------------------------------------------------------------------
     //  save args
     //-------------------------------------------------------------------------
     config = configArg;
     port = portArg;
     portText = String.valueOf(port);

     //-------------------------------------------------------------------------
     //  open the log
     //-------------------------------------------------------------------------
     StringBuffer pathname = new StringBuffer();

     try {
          //--------------------------------------------------------------------
          //  see if using log
          //--------------------------------------------------------------------
          out = null;

          String homeDirectory = config.get("homeDirectory");
          String logFile = config.get("logFile");

          if((homeDirectory == null) || (logFile == null)) return;

          //--------------------------------------------------------------------
          //  build log pathname
          //--------------------------------------------------------------------
          pathname.append(homeDirectory);
          pathname.append(File.separator);
          pathname.append(logFile);

          //--------------------------------------------------------------------
          //  open log file for appending
          //--------------------------------------------------------------------
          FileOutputStream f = new FileOutputStream(pathname.toString(),true);
          out = new PrintWriter(f);
          }
     catch(FileNotFoundException e) {
          throw new SpaceException("SpaceLog: Unable To Open Log: " + pathname.toString());
          }
     }

//------------------------------------------------------------------------------
//  println
//------------------------------------------------------------------------------
public void println(int level,String message) throws SpaceException {
     println(level,message,0,null);
     }

//------------------------------------------------------------------------------
//  println
//------------------------------------------------------------------------------
public void println(int level,String message,int port) throws SpaceException {
     println(level,message,port,null);
     }

//------------------------------------------------------------------------------
//  println
//------------------------------------------------------------------------------
public void println(int level,String message,SpacePacket packet) throws SpaceException {
     println(level,message,0,packet);
     }

//------------------------------------------------------------------------------
//  println
//------------------------------------------------------------------------------
private void println(int level,String message,int port,SpacePacket packet) throws SpaceException {
     StringBuffer sb = new StringBuffer();
     sb.append((new InvisibleDate()).getLogDateString());
     sb.append(" ");
     sb.append(levelText[level]);
     sb.append(" ");
     if(port <= 0) {
          sb.append(getDescription(packet));
          }
     else {
          sb.append(InvisibleUtil.fill(String.valueOf(port),11,' '));
          }
     sb.append(" ");
     sb.append(message);

     println(sb.toString());
     }

//------------------------------------------------------------------------------
//  println
//------------------------------------------------------------------------------
private void println(String s) {
     if(out == null) return;
     out.println(s);
     out.flush();
     }

//------------------------------------------------------------------------------
//  port/channel/serial description
//------------------------------------------------------------------------------
private String getDescription(SpacePacket packet) throws SpaceException {
     if(packet == null) return("           ");
     StringBuffer sb = new StringBuffer();
     sb.append(portText);
     sb.append(".");
     sb.append(packet.getChannel());
     sb.append(".");
     sb.append(packet.getSerial());
     return(InvisibleUtil.fill(sb.toString(),11,' '));
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public void setPort(int value) {
     port = value;
     portText = String.valueOf(port);
     }
}
