//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.io.InputStream;
import java.io.StringBufferInputStream;
import java.io.Reader;
import java.io.StringReader;

public class SpacePacket {

protected SpaceHeader                   header;
private SpaceParse                      parse;
private SpaceChannel                    channelElement;
private int                             serial;

private int                             reqno;
private StringBuffer                    response;
private boolean                         incomplete;
private boolean                         error;
private int                             frameCount;

//------------------------------------------------------------------------------
//  constuctor
//------------------------------------------------------------------------------
public SpacePacket(SpaceChannel channelElementArg,int serialArg) {
     header = null;
     parse = null;
     channelElement = channelElementArg;
     serial = serialArg;

     reqno = -1;
     response = new StringBuffer();
     incomplete = true;
     error = false;
     frameCount = 0;
     }

//------------------------------------------------------------------------------
//  get input stream
//------------------------------------------------------------------------------
public Reader getReader() {
     if(response == null) return(null);
     return((Reader)(new StringReader(response.toString())));
     }

//------------------------------------------------------------------------------
//  get input stream
//------------------------------------------------------------------------------
public InputStream getInputStream() {
     if(response == null) return(null);
     return((InputStream)(new StringBufferInputStream(response.toString())));
     }

//------------------------------------------------------------------------------
//  append
//------------------------------------------------------------------------------
public void append(String s) {
     response.append(s);
     }

//------------------------------------------------------------------------------
//  append
//------------------------------------------------------------------------------
public void append(char c) {
     response.append(c);
     }

//------------------------------------------------------------------------------
//  append
//------------------------------------------------------------------------------
public void append(char[] buffer,int length) {
     response.append(buffer,0,length);
     }

//------------------------------------------------------------------------------
//  parse
//------------------------------------------------------------------------------
public void parse() throws SpaceException {
     parse = new SpaceParse(toString());
     error = parse.isError();
     String profile = parse.getStartProfile();
     if((profile != null) && (channelElement != null)) {
          channelElement.setProfile(parse.getStartProfile());
          }
     }

//------------------------------------------------------------------------------
//  toString
//------------------------------------------------------------------------------
public String toString() {
     if(response == null) return(null);
     return(response.toString());
     }

//------------------------------------------------------------------------------
//  length
//------------------------------------------------------------------------------
public int length() {
     if(response == null) return(-1);
     return(response.length());
     }

//------------------------------------------------------------------------------
//  increment frame count
//------------------------------------------------------------------------------
public int incrementFrameCount() {
     return(frameCount++);
     }

//------------------------------------------------------------------------------
//  free
//------------------------------------------------------------------------------
public void free() {
     channelElement = null;
     response = null;
     }

//------------------------------------------------------------------------------
//  parse element methods
//------------------------------------------------------------------------------
public boolean isChallenge() throws SpaceException {
     isParseDefined();
     return(parse.isChallenge());
     }

public String getChallengeString() throws SpaceException {
     isParseDefined();
     return(parse.getChallengeString());
     }

public boolean isChallengeResponse() throws SpaceException {
     isParseDefined();
     return(parse.isChallengeResponse());
     }

public String getChallengeResponseString() throws SpaceException {
     isParseDefined();
     return(parse.getChallengeResponseString());
     }

public int getErrorCode() throws SpaceException {
     isParseDefined();
     return(parse.getErrorCode());
     }

public String getErrorCodeString() throws SpaceException {
     isParseDefined();
     return(parse.getErrorCodeString());
     }

public String getErrorString() throws SpaceException {
     isParseDefined();
     return(parse.getErrorString());
     }

public boolean isStart() throws SpaceException {
     isParseDefined();
     return(parse.isStart());
     }

public int getStartChannel() throws SpaceException {
     isParseDefined();
     return(parse.getStartChannel());
     }

public String getStartProfile() throws SpaceException {
     isParseDefined();
     return(parse.getStartProfile());
     }

public String getAuthenticator() throws SpaceException {
     isParseDefined();
     return(parse.getAuthenticator());
     }

private void isParseDefined() throws SpaceException {
     if(parse == null) {
          throw new SpaceException("SpacePacket: No Parsed Results");
          }
     }

public String[] getProfiles() throws SpaceException {
     isParseDefined();
     return(parse.getProfiles());
     }

public String[] getNames() throws SpaceException {
     isParseDefined();
     return(parse.getNames());
     }

public String getLocalize() throws SpaceException {
     isParseDefined();
     return(parse.getLocalize());
     }

public String getFeatures() throws SpaceException {
     isParseDefined();
     return(parse.getFeatures());
     }

//------------------------------------------------------------------------------
//  channel element methods
//------------------------------------------------------------------------------
public long getSeqno() throws SpaceException {
     isChannelDefined();
     return(channelElement.getSeqno());
     }

public int getChannel() throws SpaceException {
     isChannelDefined();
     return(channelElement.getChannel());
     }

public String getMime() throws SpaceException {
     isChannelDefined();
     return(channelElement.getMime());
     }

public String getProfile() throws SpaceException {
     isChannelDefined();
     return(channelElement.getProfile());
     }

private void isChannelDefined() throws SpaceException {
     if(channelElement == null) {
          throw new SpaceException("SpacePacket: No Channel Open");
          }
     }

//------------------------------------------------------------------------------
//  system error
//------------------------------------------------------------------------------
public void setError(int errorCode,String errorString) {
     error = true;
     parse = new SpaceParse();
     parse.setError(true);
     parse.setErrorCode(errorCode);
     parse.setErrorCodeString(String.valueOf(errorCode));
     parse.setErrorString(errorString);
     }

//------------------------------------------------------------------------------
//  header types
//------------------------------------------------------------------------------
public boolean isResponse() { return(header.isResponse()); }
public boolean isRequest() { return(header.isRequest()); }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public boolean isComplete() { return(!(incomplete)); }
public boolean isIncomplete() { return(incomplete); }
public boolean isError() { return(error); }

public void setHeader(SpaceHeader value) { header = value; }
public void setParse(SpaceParse value) { parse = value; }
public void setChannelElement(SpaceChannel value) { channelElement = value; }
public void setIncomplete(boolean value) { incomplete = value; }
public void setError(boolean value) { error = value; }
public void setReqno(int value) { reqno = value; }

public SpaceHeader getHeader() { return(header); }
public SpaceParse getParse() { return(parse); }
public SpaceChannel getChannelElement() { return(channelElement); }
public int getSerial() { return(serial); }
public int getReqno() { return(reqno); }
public boolean getIncomplete() { return(incomplete); }
public boolean getError() { return(error); }
public int getFrameCount() { return(frameCount); }
}
