//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceParse {

private boolean                         challenge;
private String                          challengeString;
private boolean                         challengeResponse;
private String                          challengeResponseString;
private boolean                         error;
private int                             errorCode;
private String                          errorCodeString;
private String                          errorString;
private int                             startChannel;
private String                          startProfile;
private String                          authenticator;
private String                          response;
private String[]                        profiles;
private String[]                        names;
private String                          localize;
private String                          features;

//------------------------------------------------------------------------------
//  constuctor
//------------------------------------------------------------------------------
public SpaceParse() {
     init();
     }

//------------------------------------------------------------------------------
//  constuctor with response
//------------------------------------------------------------------------------
public SpaceParse(String s) throws SpaceException {
     init();
     parse(s);
     }

//------------------------------------------------------------------------------
//  init
//------------------------------------------------------------------------------
private void init() {
     challenge = false;
     challengeString = null;
     challengeResponse = false;
     challengeResponseString = null;
     error = false;
     errorCode = 0;
     errorCodeString = null;
     errorString = null;
     startChannel = -1;
     startProfile = null;
     authenticator = null;
     }

//------------------------------------------------------------------------------
//  very simple and specific parser
//------------------------------------------------------------------------------
public void parse(String s) throws SpaceException {
     //-------------------------------------------------------------------------
     //  mark as an error if null or empty
     //-------------------------------------------------------------------------
     if((s == null) || (s.length() == 0)) {
          error = true;
          errorCode = 500;
          errorCodeString = "500";
          errorString = "Premature EOF";
          return;
          }

     //-------------------------------------------------------------------------
     //  look for an error
     //-------------------------------------------------------------------------
     int index;
     int indexend;

     if((index = s.indexOf("<error")) != -1) {
          //--------------------------------------------------------------------
          //  get error code
          //--------------------------------------------------------------------
          errorCodeString = getAttribute(s,"code","Error Code",index,true);
          errorCode = InvisibleUtil.parseInt(errorCodeString);

          //--------------------------------------------------------------------
          //  get localization if appropriate
          //--------------------------------------------------------------------
          localize = getAttribute(s,"xml:lang","Localize",index,false);

          //--------------------------------------------------------------------
          //  get features if appropriate
          //--------------------------------------------------------------------
          features = getAttribute(s,"features","Features",index,false);

          //--------------------------------------------------------------------
          //  see if text description
          //--------------------------------------------------------------------
          if((indexend = s.indexOf("</error>")) != -1) {
               index = s.indexOf(">",index);
               errorString = s.substring(++index,indexend);
               errorString.replace('\r',' ');
               errorString.replace('\n',' ');
               errorString = errorString.trim();
               }

          error = true;

          return;
          }

     //-------------------------------------------------------------------------
     //  look for a authenticator/challenge
     //-------------------------------------------------------------------------
     if(s.startsWith("<profile")) {
          //--------------------------------------------------------------------
          //  unencoded challenge
          //--------------------------------------------------------------------
          if((index = s.indexOf("<challenge>")) != -1) {
               index += 11; //length of <challenge>
               indexend = s.indexOf("</challenge>",index);
               if(indexend == -1) {
                    throw new SpaceException("SpaceParse: Unrecognized Challenge: " + s);
                    }
               challengeString = s.substring(index,indexend);
               challenge = true;
               } 

          //--------------------------------------------------------------------
          //  encoded challenge
          //--------------------------------------------------------------------
          if((index = s.indexOf("<blob>")) != -1) {
               index += 6; //length of <blob>
               indexend = s.indexOf("</blob>",index);
               if(indexend == -1) {
                    throw new SpaceException("SpaceParse: Unrecognized Challenge: " + s);
                    }
               challengeString = s.substring(index,indexend);
               challenge = true;
               } 
 
          return;
          }

     //-------------------------------------------------------------------------
     //  look for a greeting
     //-------------------------------------------------------------------------
     if(s.startsWith("<greeting")) {
          try {
               //---------------------------------------------------------------
               //  parse the greeting
               //---------------------------------------------------------------
               InvisibleTokenizer parse = new InvisibleTokenizer(s);

               //---------------------------------------------------------------
               //  accumulate attributes in these vectors
               //---------------------------------------------------------------
               InvisibleVector profileVector = new InvisibleVector();

               //---------------------------------------------------------------
               //  parse results
               //---------------------------------------------------------------
               while(parse.next()) {
                    //----------------------------------------------------------
                    //  get the key/value pair
                    //----------------------------------------------------------
                    String key = parse.getKey().toLowerCase();
                    String value = parse.getValue();

                    //----------------------------------------------------------
                    //  profile uri
                    //----------------------------------------------------------
                    if((parse.isAttribute()) && (key.equals("uri"))) {
                         profileVector.set(value);
                         continue;
                         }

                    //----------------------------------------------------------
                    //  localize
                    //----------------------------------------------------------
                    if((parse.isAttribute()) && (key.equals("localize"))) {
                         localize = value;
                         continue;
                         }

                    //----------------------------------------------------------
                    //  features
                    //----------------------------------------------------------
                    if((parse.isAttribute()) && (key.equals("features"))) {
                         features = value;
                         continue;
                         }
                    }

               //---------------------------------------------------------------
               //  save the profiles
               //---------------------------------------------------------------
               int size = profileVector.size();
               profiles = new String[size];
               for(int k=0;k<size;++k) profiles[k] = profileVector.getString(k);

               return;
               }
          catch(InvisibleException e) {
               throw new SpaceException("SpaceParse: Can Not Parse Greeting: "+e.toString());
               }
          }

     //-------------------------------------------------------------------------
     //  look for a channel start
     //-------------------------------------------------------------------------
     if(s.startsWith("<start")) {
          index = 6;
          startChannel = InvisibleUtil.parseInt(getAttribute(s,"number","Start Channel Number",index,true));
          startProfile = getAttribute(s,"uri","Start Channel Profile",index,true);

          //--------------------------------------------------------------------
          //  encoded authenticator
          //--------------------------------------------------------------------
          if((index = s.indexOf("<blob>")) != -1) {
               index +=  6;
               indexend = s.indexOf("</blob>",index);
               if(indexend == -1) {
                    throw new SpaceException("SpaceParse: Unrecognized Authenticator: " + s);
                    }
               authenticator = s.substring(index,indexend);
               }

          return;
          }

     //-------------------------------------------------------------------------
     //  look for a challenge response--must be before next response
     //-------------------------------------------------------------------------
     if(s.startsWith("<blob>")) {
          index = 6;
          if((indexend = s.indexOf("</blob>",index)) == -1) {
               throw new SpaceException("SpaceParse: Incomplete Challenge Response");
               }

          challengeResponseString = s.substring(index,indexend);
          challengeResponse = true;
          return;
          }

     //-------------------------------------------------------------------------
     //  look for a response
     //-------------------------------------------------------------------------
     if(s.startsWith("<response")) {
          //--------------------------------------------------------------------
          //  get the root element for each block
          //--------------------------------------------------------------------
          InvisibleVector nameVector = new InvisibleVector();

          if((index = s.indexOf("<answers")) == -1) {
               throw new SpaceException("SpaceParse: No Answers");
               }

          if((index = s.indexOf("<",++index)) == -1) {
               throw new SpaceException("SpaceParse: Incomplete Answer");
               }

          if((indexend = s.indexOf(" ",index+1)) == -1) {
               throw new SpaceException("SpaceParse: Incomplete Element");
               }

          String element = s.substring(index,++indexend);
          index = indexend;

          //--------------------------------------------------------------------
          //  extract names for each root element
          //--------------------------------------------------------------------
          while(true) {
               if((index = s.indexOf("name=",index)) == -1) break;
               index += 5; //length of name=
               char delimiter = s.charAt(index++);
               if((indexend = s.indexOf(delimiter,index)) == -1) {
                    throw new SpaceException("SpaceParse: Incomplete Name");
                    }
               String name = s.substring(index,indexend);
               nameVector.set(name);
               if((index = s.indexOf(element,indexend)) == -1) break;
               }

          int size = nameVector.size();
          names = new String[size];
          for(int k=0;k<size;++k) names[k] = nameVector.getString(k);

          return;
          }
     }

//------------------------------------------------------------------------------
//  get attribute
//------------------------------------------------------------------------------
private String getAttribute(String s,String name,String errorMessage,int index,boolean exceptionIfMissing) throws SpaceException {
     String nameEquals = name + "=";
     if((index = s.indexOf(nameEquals,index)) == -1) {
          if(exceptionIfMissing) {
               throw new SpaceException("SpaceParse: "+errorMessage+" Missing");
               }

          return(null);
          }

     index += nameEquals.length();
     char delimiter = s.charAt(index++);

     int indexend;

     if((indexend = s.indexOf(delimiter,index)) == -1) {
          throw new SpaceException("SpaceParse: "+errorMessage+" Incomplete");
          }

     return(s.substring(index,indexend));
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public boolean isChallenge() { return(challenge); }
public boolean isChallengeResponse() { return(challengeResponse); }
public boolean isError() { return(error); }
public boolean isStart() { return(startChannel != -1); }

public void setError(boolean value) { error = value; }
public void setErrorCode(int value) { errorCode = value; }
public void setErrorCodeString(String value) { errorCodeString = value; }
public void setErrorString(String value) { errorString = value; }

public boolean getChallenge() { return(challenge); }
public String getChallengeString() { return(challengeString); }
public boolean getChallengeResponse() { return(challengeResponse); }
public String getChallengeResponseString() { return(challengeResponseString); }
public boolean getError() { return(error); }
public int getErrorCode() { return(errorCode); }
public String getErrorCodeString() { return(errorCodeString); }
public String getErrorString() { return(errorString); }
public int getStartChannel() { return(startChannel); }
public String getStartProfile() { return(startProfile); }
public String getAuthenticator() { return(authenticator); }
public String[] getProfiles() { return(profiles); }
public String[] getNames() { return(names); }
public String getLocalize() { return(localize); }
public String getFeatures() { return(features); }
}
