//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceRequestQueue {

InvisibleVector                         waiting;
InvisibleVectorTable                    active;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceRequestQueue() {
     waiting = new InvisibleVector();
     active = new InvisibleVectorTable();
     }

//------------------------------------------------------------------------------
//  set a request
//------------------------------------------------------------------------------
synchronized public void set(SpacePacket request) {
     //-------------------------------------------------------------------------
     //  add the request to the queue
     //-------------------------------------------------------------------------
     waiting.set(request);
     }

//------------------------------------------------------------------------------
//  get first request
//------------------------------------------------------------------------------
synchronized public SpaceRequest get() {
     //-------------------------------------------------------------------------
     //  see if there is a waiting request
     //-------------------------------------------------------------------------
     int size = waiting.size();

     if(size == 0) return(null);

     //-------------------------------------------------------------------------
     //  find the most available request
     //-------------------------------------------------------------------------
     for(int k=0;k<size;++k) {
          //--------------------------------------------------------------------
          //  get the request from the waiting queue
          //--------------------------------------------------------------------
          SpaceRequest request = (SpaceRequest)(waiting.get(k));

          //--------------------------------------------------------------------
          //  ignore it if there is a pending request on that channel
          //--------------------------------------------------------------------
          SpaceChannel channelElement = request.getChannelElement();

          String channel = String.valueOf(channelElement.getChannel());

          if(active.get(channel) != null) {
               //---------------------------------------------------------------
               //  channel already active
               //---------------------------------------------------------------
               continue;
               }

          //--------------------------------------------------------------------
          //  remove the request from the waiting queue
          //--------------------------------------------------------------------
          waiting.remove(k);

          //--------------------------------------------------------------------
          //  mark the request active
          //--------------------------------------------------------------------
          active.set(channel,request);

          //--------------------------------------------------------------------
          //  return the request
          //--------------------------------------------------------------------
          return(request);
          }

     return(null);
     }

//------------------------------------------------------------------------------
//  mark channel available
//------------------------------------------------------------------------------
synchronized public void free(int serial) {
     //-------------------------------------------------------------------------
     //  get the active requests
     //-------------------------------------------------------------------------
     InvisibleVector requests = active.getTableObject();

     //-------------------------------------------------------------------------
     //  find the channel associated with the serial
     //-------------------------------------------------------------------------
     int size = requests.size();

     for(int k=0;k<size;++k) {
          SpaceRequest request = (SpaceRequest)(active.get(k));

          if(request.getSerial() == serial) {
               //---------------------------------------------------------------
               //  found the channel
               //---------------------------------------------------------------
               String channel = String.valueOf(request.getChannelElement().getChannel());

               //---------------------------------------------------------------
               //  remove the request from the active queue
               //---------------------------------------------------------------
               active.remove(channel);

               return;
               }
          }
     }
}
