//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
public class SpaceSerialCache {

private final static int                START = 32;
private final static int                STEP = 16;
private final static int                MAX = 32768;

private long                            serialCache[];
private int                             max;
private long                            timeout;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceSerialCache() {
     this(START);
     }

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
private SpaceSerialCache(int maxArg) {
     if(maxArg > MAX) {
          max = MAX;
          }
     else {
          max = maxArg;
          }
     timeout = SpaceSocket.defaultTimeout;
     serialCache = new long[max];
     for(int k=0;k<max;++k) serialCache[k] = 0L;
     }

//------------------------------------------------------------------------------
//  get available serial number (not used or timed out)
//------------------------------------------------------------------------------
synchronized public int get() throws SpaceException {
     //-------------------------------------------------------------------------
     //  sequentially scan for unassigned or expired serial
     //-------------------------------------------------------------------------
     long now = System.currentTimeMillis();

     for(int k=1;k<max;++k) {
          long l = serialCache[k];

          if((l == 0) || ((l + timeout) < now)) {
               serialCache[k] = now;
               return(k);
               }
          }

     //-------------------------------------------------------------------------
     //  need to expand
     //-------------------------------------------------------------------------
     int maxExpanded = max + STEP;

     if(maxExpanded > MAX) {
          throw new SpaceException("SpaceSerialCache: Too Many Connections");
          }

     long serialCacheExpanded[] = new long[maxExpanded];
     for(int k=0;k<max;++k) serialCacheExpanded[k] = serialCache[k];
     for(int k=max;k<maxExpanded;++k) serialCacheExpanded[k] = 0L;
     serialCache = serialCacheExpanded;

     int index = max;
     max = maxExpanded;
     serialCache[index] = now;

     return(index);
     }

//------------------------------------------------------------------------------
//  set serial number
//------------------------------------------------------------------------------
synchronized public void set(int serial) {
     serialCache[serial] = 0L;
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public void setTimeout(long value) { timeout = value; }

public long getTimeout() { return(timeout); }
}
