//------------------------------------------------------------------------------
//  Copyright (c) 2000, Invisible Worlds, Inc. All rights reserved.
// 
//  The contents of this file are subject to the Blocks Public License (the 
//  "License"); you may not use this file except in compliance with the License.  
//  You may obtain a copy of the License at http://www.invisible.net/ 
// 
//  Software distributed under the License is distributed on an "AS IS" basis,  
//  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for  
//  the specific language governing rights and limitations under the License. 
//------------------------------------------------------------------------------
import java.io.File;

public class SpaceUserCache extends InvisibleProperties {

String                                  authenticator;
String                                  propertyClass;

//------------------------------------------------------------------------------
//  constructor
//------------------------------------------------------------------------------
public SpaceUserCache() {
     super();
     }

//------------------------------------------------------------------------------
//  load the users properties file
//------------------------------------------------------------------------------
public void load(SpaceConfig config,SpaceProfile profile) throws SpaceException {
     try {
          load(getPathname(config,profile));
          }
     catch(InvisibleException e) {
          throw new SpaceException("SpaceUserCache: " + e.toString());
          }
     }

//------------------------------------------------------------------------------
//  store the user properties file
//------------------------------------------------------------------------------
public void store(SpaceConfig config,SpaceProfile profile) throws SpaceException {
     try {
          store(getPathname(config,profile),"Bxxd Usernames");
          }
     catch(InvisibleException e) {
          throw new SpaceException("SpaceUserCache: " + e.toString());
          }
     }

//------------------------------------------------------------------------------
//  get a name from supplied authenticator/propertyClass/name
//------------------------------------------------------------------------------
public String get(String authenticator,String propertyClass,String name) {
     return(super.get(getName(authenticator,propertyClass,name)));
     }

//------------------------------------------------------------------------------
//  get a name from supplied propertyClass/name
//------------------------------------------------------------------------------
public String get(String propertyClass,String name) {
     return(super.get(getName(authenticator,propertyClass,name)));
     }

//------------------------------------------------------------------------------
//  get a name from supplied name
//------------------------------------------------------------------------------
public String get(String name) {
     return(super.get(getName(authenticator,propertyClass,name)));
     }

//------------------------------------------------------------------------------
//  set a name from supplied authenticator/propertyClass/name
//------------------------------------------------------------------------------
public void set(String authenticator,String propertyClass,String name,String value) {
     super.set(getName(authenticator,propertyClass,name),value);
     }

//------------------------------------------------------------------------------
//  set a name from supplied propertyClass/name
//------------------------------------------------------------------------------
public void set(String propertyClass,String name,String value) {
     super.set(getName(authenticator,propertyClass,name),value);
     }

//------------------------------------------------------------------------------
//  set a name from supplied name
//------------------------------------------------------------------------------
public void set(String name,String value) {
     super.set(getName(authenticator,propertyClass,name),value);
     }

//------------------------------------------------------------------------------
//  get property name from the supplied authenticator/propertyClass/name
//------------------------------------------------------------------------------
private String getName(String authenticator,String propertyClass,String name) {
     StringBuffer fullname = new StringBuffer();
     fullname.append("username.");
     fullname.append(authenticator);
     fullname.append(".");
     fullname.append(propertyClass);
     fullname.append(".");
     fullname.append(name);

     return(fullname.toString());
     }

//------------------------------------------------------------------------------
//  get pathname for user properties
//------------------------------------------------------------------------------
private String getPathname(SpaceConfig config,SpaceProfile profile) {
     StringBuffer pathname = new StringBuffer();
     pathname.append(config.get("homeDirectory"));
     pathname.append(File.separator);
     pathname.append(profile.get("authDirectory"));
     pathname.append(config.get("port"));
     pathname.append(".properties");

     return(pathname.toString());
     }

//------------------------------------------------------------------------------
//  bean property access methods
//------------------------------------------------------------------------------
public void setAuthenticator(String value) { authenticator = value; }
public void setPropertyClass(String value) { propertyClass = value; }
}
